
from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    EnvironmentExtensions,
    EnvironmentExtensionsBulkRequest,
    EnvironmentExtensionsBulkResponse,
    EnvironmentExtensionsQueryConfig,
    EnvironmentExtensionsQueryResponse,
)


class EnvironmentExtensionsService(BaseService):

    @cast_models
    def get_environment_extensions(self, id_: str) -> Union[EnvironmentExtensions, str]:
        """Retrieves the extension values for the environment having the specified ID (except for encrypted values).

        :param id_: The ID of the object. This can be either of the following:
         1. The value of `environmentId`.
         2. A conceptual ID synthesized from the environment ID (`environmentId`) and the ID of the multi-install integration pack to which the extension values apply (`extensionGroupId`).
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentExtensions, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentExtensions/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentExtensions._unmap(response)
        if content == "application/xml":
            return EnvironmentExtensions._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_environment_extensions(
        self, id_: str, request_body: EnvironmentExtensions = None
    ) -> Union[EnvironmentExtensions, str]:
        """Updates the extension values for the environment having the specified ID. When updating extension values, you must perform either a partial update to update only those extension values requiring modification in the request, or a full update to update the full set of environment extensions in a single request. A partial update is typically recommended because it results in smaller payloads and more targeted updates.

         >**Warning:** The UPDATE operation does not support running muliple map extensions requests concurrently. Some map extensions might not get updated properly.

         #### Performing a partial update

        To perform a **partial update**, set `partial` to true and then provide only the extension fields and values that you wish to update in the request.

        >**Note:** For cross reference tables, you can update a single cross reference table. However, you must provide all values for the entire table. You cannot update individual rows within a table.
        >
        > - For process property components, you can update a single process property component but you must provide the values for all properties in the component.

         #### Performing a full update

        To perform a **full update**, set `partial` to false and then provide all the environment extension fields and values in the request, regardless if you wish to change only some values but not all.

        >**Caution:** Values not included in the request are reset to use their default values. If you omit the partial attribute, the behavior defaults to a full update.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentExtensions, optional
        :param id_: The ID of the object. This can be either of the following:
         1. The value of environmentId.
         2. A conceptual ID synthesized from the environment ID (environmentId) and the ID of the multi-install integration pack to which the extension values apply (extensionGroupId).
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentExtensions, str]
        """

        Validator(EnvironmentExtensions).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentExtensions/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentExtensions._unmap(response)
        if content == "application/xml":
            return EnvironmentExtensions._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_environment_extensions(
        self, request_body: EnvironmentExtensionsBulkRequest = None
    ) -> Union[EnvironmentExtensionsBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentExtensionsBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentExtensionsBulkResponse, str]
        """

        Validator(EnvironmentExtensionsBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentExtensions/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentExtensionsBulkResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentExtensionsBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_environment_extensions(
        self, request_body: EnvironmentExtensionsQueryConfig = None
    ) -> Union[EnvironmentExtensionsQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentExtensionsQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentExtensionsQueryResponse, str]
        """

        Validator(EnvironmentExtensionsQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentExtensions/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentExtensionsQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentExtensionsQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_environment_extensions(
        self, request_body: str
    ) -> Union[EnvironmentExtensionsQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentExtensionsQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentExtensions/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentExtensionsQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentExtensionsQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class Privilege(BaseModel):
    """Privilege

    :param name: name
    :type name: str
    """

    def __init__(self, name: str, **kwargs):
        """Privilege

        :param name: name
        :type name: str
        """
        self.name = name
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    OrganizationComponent,
    OrganizationComponentBulkRequest,
    OrganizationComponentBulkResponse,
    OrganizationComponentQueryConfig,
    OrganizationComponentQueryResponse,
)


class OrganizationComponentService(BaseService):

    @cast_models
    def create_organization_component(
        self, request_body: OrganizationComponent = None
    ) -> Union[OrganizationComponent, str]:
        """The CREATE operation creates an Organization Component object with the specified component name.

         The request body requires the `componentName` field. If you omit the `folderName` field, it requires the `folderId` field — and vice versa. If you omit the `componentID` field, it assigns the value when you create the component. If you omit the `folderID` field, it assigns the value when you create the component.

        :param request_body: The request body., defaults to None
        :type request_body: OrganizationComponent, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[OrganizationComponent, str]
        """

        Validator(OrganizationComponent).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return OrganizationComponent._unmap(response)
        if content == "application/xml":
            return OrganizationComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_organization_component(self, id_: str) -> Union[OrganizationComponent, str]:
        """The GET operation returns a single Organization Component object based on the supplied ID. A GET operation specifying the ID of a deleted Organization Component retrieves the component. In the component, the deleted field’s value is *true*.

        :param id_: Organization component ID
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[OrganizationComponent, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return OrganizationComponent._unmap(response)
        if content == "application/xml":
            return OrganizationComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_organization_component(
        self, id_: str, request_body: OrganizationComponent = None
    ) -> Union[OrganizationComponent, str]:
        """The UPDATE operation overwrites the Organization Component object with the specified component ID. An UPDATE operation specifying the ID of a deleted Organization component restores the component to a non-deleted state, assuming the request is otherwise valid.

        :param request_body: The request body., defaults to None
        :type request_body: OrganizationComponent, optional
        :param id_: Organization component ID
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[OrganizationComponent, str]
        """

        Validator(OrganizationComponent).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return OrganizationComponent._unmap(response)
        if content == "application/xml":
            return OrganizationComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_organization_component(self, id_: str) -> None:
        """The DELETE operation deletes the Organization Component object with the specified component ID. A DELETE operation specifying the ID of a deleted Organization component returns a false response. If the component is deleted successfully, the response is `true`.

        :param id_: ID of the Organization component you are attempting to delete.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_organization_component(
        self, request_body: OrganizationComponentBulkRequest = None
    ) -> Union[str, OrganizationComponentBulkResponse]:
        """The bulk GET operation returns multiple Account objects based on the supplied account IDs, to a maximum of 100. To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: OrganizationComponentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[str, OrganizationComponentBulkResponse]
        """

        Validator(OrganizationComponentBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/xml":
            return OrganizationComponentBulkResponse._unmap(parse_xml_to_dict(response))
        if content == "application/json":
            return OrganizationComponentBulkResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_organization_component(
        self, request_body: OrganizationComponentQueryConfig = None
    ) -> Union[OrganizationComponentQueryResponse, str]:
        """- Only the LIKE operator is allowed with a name filter. Likewise, only the EQUALS operator is permitted with a contactName, email, or phone filter.

         -   If the QUERY request includes multiple filters, you can connect the filters with a logical AND operator — the query does not support the logical OR operator .

         -   The QUERY results omit the folderName field.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: OrganizationComponentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[OrganizationComponentQueryResponse, str]
        """

        Validator(OrganizationComponentQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return OrganizationComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return OrganizationComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_organization_component(
        self, request_body: str
    ) -> Union[OrganizationComponentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[OrganizationComponentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/OrganizationComponent/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return OrganizationComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return OrganizationComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class TradingPartnerComponentSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar LIKE: "LIKE"
    :vartype LIKE: str
    :cvar NOTEQUALS: "NOT_EQUALS"
    :vartype NOTEQUALS: str
    :cvar ISNULL: "IS_NULL"
    :vartype ISNULL: str
    :cvar ISNOTNULL: "IS_NOT_NULL"
    :vartype ISNOTNULL: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    :cvar CONTAINS: "CONTAINS"
    :vartype CONTAINS: str
    :cvar NOTCONTAINS: "NOT_CONTAINS"
    :vartype NOTCONTAINS: str
    """

    EQUALS = "EQUALS"
    LIKE = "LIKE"
    NOTEQUALS = "NOT_EQUALS"
    ISNULL = "IS_NULL"
    ISNOTNULL = "IS_NOT_NULL"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"
    CONTAINS = "CONTAINS"
    NOTCONTAINS = "NOT_CONTAINS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                TradingPartnerComponentSimpleExpressionOperator._member_map_.values(),
            )
        )


class TradingPartnerComponentSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar NAME: "name"
    :vartype NAME: str
    :cvar CLASSIFICATION: "classification"
    :vartype CLASSIFICATION: str
    :cvar STANDARD: "standard"
    :vartype STANDARD: str
    :cvar IDENTIFIER: "identifier"
    :vartype IDENTIFIER: str
    :cvar AS2: "as2"
    :vartype AS2: str
    :cvar DISK: "disk"
    :vartype DISK: str
    :cvar FTP: "ftp"
    :vartype FTP: str
    :cvar MLLP: "mllp"
    :vartype MLLP: str
    :cvar SFTP: "sftp"
    :vartype SFTP: str
    :cvar HTTP: "http"
    :vartype HTTP: str
    :cvar OFTP: "oftp"
    :vartype OFTP: str
    """

    NAME = "name"
    CLASSIFICATION = "classification"
    STANDARD = "standard"
    IDENTIFIER = "identifier"
    AS2 = "as2"
    DISK = "disk"
    FTP = "ftp"
    MLLP = "mllp"
    SFTP = "sftp"
    HTTP = "http"
    OFTP = "oftp"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                TradingPartnerComponentSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class TradingPartnerComponentSimpleExpression(BaseModel):
    """TradingPartnerComponentSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: TradingPartnerComponentSimpleExpressionOperator
    :param property: property
    :type property: TradingPartnerComponentSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: TradingPartnerComponentSimpleExpressionOperator,
        property: TradingPartnerComponentSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """TradingPartnerComponentSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: TradingPartnerComponentSimpleExpressionOperator
        :param property: property
        :type property: TradingPartnerComponentSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, TradingPartnerComponentSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, TradingPartnerComponentSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .private_certificate import PrivateCertificate
from .public_certificate import PublicCertificate


class MdnDigestAlg(Enum):
    """An enumeration representing different categories.

    :cvar SHA1: "SHA1"
    :vartype SHA1: str
    :cvar SHA224: "SHA224"
    :vartype SHA224: str
    :cvar SHA256: "SHA256"
    :vartype SHA256: str
    :cvar SHA384: "SHA384"
    :vartype SHA384: str
    :cvar SHA512: "SHA512"
    :vartype SHA512: str
    """

    SHA1 = "SHA1"
    SHA224 = "SHA224"
    SHA256 = "SHA256"
    SHA384 = "SHA384"
    SHA512 = "SHA512"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, MdnDigestAlg._member_map_.values()))


class Synchronous(Enum):
    """An enumeration representing different categories.

    :cvar SYNC: "sync"
    :vartype SYNC: str
    :cvar ASYNC: "async"
    :vartype ASYNC: str
    """

    SYNC = "sync"
    ASYNC = "async"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Synchronous._member_map_.values()))


@JsonMap(
    {
        "external_url": "externalURL",
        "mdn_client_ssl_cert": "mdnClientSSLCert",
        "mdn_digest_alg": "mdnDigestAlg",
        "mdn_ssl_cert": "mdnSSLCert",
        "request_mdn": "requestMDN",
        "use_external_url": "useExternalURL",
        "use_ssl": "useSSL",
    }
)
class As2MdnOptions(BaseModel):
    """As2MdnOptions

    :param external_url: external_url
    :type external_url: str
    :param mdn_client_ssl_cert: mdn_client_ssl_cert
    :type mdn_client_ssl_cert: PrivateCertificate
    :param mdn_digest_alg: mdn_digest_alg, defaults to None
    :type mdn_digest_alg: MdnDigestAlg, optional
    :param mdn_ssl_cert: mdn_ssl_cert
    :type mdn_ssl_cert: PublicCertificate
    :param request_mdn: request_mdn, defaults to None
    :type request_mdn: bool, optional
    :param signed: signed, defaults to None
    :type signed: bool, optional
    :param synchronous: synchronous, defaults to None
    :type synchronous: Synchronous, optional
    :param use_external_url: use_external_url, defaults to None
    :type use_external_url: bool, optional
    :param use_ssl: use_ssl, defaults to None
    :type use_ssl: bool, optional
    """

    def __init__(
        self,
        external_url: str,
        mdn_client_ssl_cert: PrivateCertificate,
        mdn_ssl_cert: PublicCertificate,
        mdn_digest_alg: MdnDigestAlg = SENTINEL,
        request_mdn: bool = SENTINEL,
        signed: bool = SENTINEL,
        synchronous: Synchronous = SENTINEL,
        use_external_url: bool = SENTINEL,
        use_ssl: bool = SENTINEL,
        **kwargs,
    ):
        """As2MdnOptions

        :param external_url: external_url
        :type external_url: str
        :param mdn_client_ssl_cert: mdn_client_ssl_cert
        :type mdn_client_ssl_cert: PrivateCertificate
        :param mdn_digest_alg: mdn_digest_alg, defaults to None
        :type mdn_digest_alg: MdnDigestAlg, optional
        :param mdn_ssl_cert: mdn_ssl_cert
        :type mdn_ssl_cert: PublicCertificate
        :param request_mdn: request_mdn, defaults to None
        :type request_mdn: bool, optional
        :param signed: signed, defaults to None
        :type signed: bool, optional
        :param synchronous: synchronous, defaults to None
        :type synchronous: Synchronous, optional
        :param use_external_url: use_external_url, defaults to None
        :type use_external_url: bool, optional
        :param use_ssl: use_ssl, defaults to None
        :type use_ssl: bool, optional
        """
        self.external_url = external_url
        self.mdn_client_ssl_cert = self._define_object(
            mdn_client_ssl_cert, PrivateCertificate
        )
        if mdn_digest_alg is not SENTINEL:
            self.mdn_digest_alg = self._enum_matching(
                mdn_digest_alg, MdnDigestAlg.list(), "mdn_digest_alg"
            )
        self.mdn_ssl_cert = self._define_object(mdn_ssl_cert, PublicCertificate)
        if request_mdn is not SENTINEL:
            self.request_mdn = request_mdn
        if signed is not SENTINEL:
            self.signed = signed
        if synchronous is not SENTINEL:
            self.synchronous = self._enum_matching(
                synchronous, Synchronous.list(), "synchronous"
            )
        if use_external_url is not SENTINEL:
            self.use_external_url = use_external_url
        if use_ssl is not SENTINEL:
            self.use_ssl = use_ssl
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..folder import FolderService
from ...models import (
    Folder,
    FolderBulkResponse,
    FolderBulkRequest,
    FolderQueryResponse,
    FolderQueryConfig,
)


class FolderServiceAsync(FolderService):
    """
    Async Wrapper for FolderServiceAsync
    """

    def create_folder(
        self, request_body: Folder = None
    ) -> Awaitable[Union[Folder, str]]:
        return to_async(super().create_folder)(request_body)

    def get_folder(self, id_: str) -> Awaitable[Union[Folder, str]]:
        return to_async(super().get_folder)(id_)

    def update_folder(
        self, id_: str, request_body: Folder = None
    ) -> Awaitable[Union[Folder, str]]:
        return to_async(super().update_folder)(id_, request_body)

    def delete_folder(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_folder)(id_)

    def bulk_folder(
        self, request_body: FolderBulkRequest = None
    ) -> Awaitable[Union[FolderBulkResponse, str]]:
        return to_async(super().bulk_folder)(request_body)

    def query_folder(
        self, request_body: FolderQueryConfig = None
    ) -> Awaitable[Union[FolderQueryResponse, str]]:
        return to_async(super().query_folder)(request_body)

    def query_more_folder(
        self, request_body: str
    ) -> Awaitable[Union[FolderQueryResponse, str]]:
        return to_async(super().query_more_folder)(request_body)




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .nodes import Nodes


class Capabilities(Enum):
    """An enumeration representing different categories.

    :cvar GATEWAY: "GATEWAY"
    :vartype GATEWAY: str
    :cvar BROKER: "BROKER"
    :vartype BROKER: str
    """

    GATEWAY = "GATEWAY"
    BROKER = "BROKER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Capabilities._member_map_.values()))


class AtomStatus(Enum):
    """An enumeration representing different categories.

    :cvar UNKNOWN: "UNKNOWN"
    :vartype UNKNOWN: str
    :cvar ONLINE: "ONLINE"
    :vartype ONLINE: str
    :cvar WARNING: "WARNING"
    :vartype WARNING: str
    :cvar OFFLINE: "OFFLINE"
    :vartype OFFLINE: str
    """

    UNKNOWN = "UNKNOWN"
    ONLINE = "ONLINE"
    WARNING = "WARNING"
    OFFLINE = "OFFLINE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AtomStatus._member_map_.values()))


class AtomType(Enum):
    """An enumeration representing different categories.

    :cvar CLOUD: "CLOUD"
    :vartype CLOUD: str
    :cvar MOLECULE: "MOLECULE"
    :vartype MOLECULE: str
    :cvar ATOM: "ATOM"
    :vartype ATOM: str
    """

    CLOUD = "CLOUD"
    MOLECULE = "MOLECULE"
    ATOM = "ATOM"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AtomType._member_map_.values()))


@JsonMap(
    {
        "cloud_id": "cloudId",
        "cloud_molecule_id": "cloudMoleculeId",
        "cloud_molecule_name": "cloudMoleculeName",
        "cloud_name": "cloudName",
        "cloud_owner_name": "cloudOwnerName",
        "created_by": "createdBy",
        "current_version": "currentVersion",
        "date_installed": "dateInstalled",
        "force_restart_time": "forceRestartTime",
        "host_name": "hostName",
        "id_": "id",
        "instance_id": "instanceId",
        "is_cloud_attachment": "isCloudAttachment",
        "purge_history_days": "purgeHistoryDays",
        "purge_immediate": "purgeImmediate",
        "type_": "type",
        "status_detail": "statusDetail",
    }
)
class Atom(BaseModel):
    """Atom

    :param capabilities: capabilities, defaults to None
    :type capabilities: List[Capabilities], optional
    :param cloud_id: \(For Runtimes attached to Runtime clouds\) A unique ID assigned by the system to the Runtime cloud., defaults to None
    :type cloud_id: str, optional
    :param cloud_molecule_id: ID of the Runtime cloud cluster to which the Cloud Attachment is assigned. This field is populated only for Cloud Attachments., defaults to None
    :type cloud_molecule_id: str, optional
    :param cloud_molecule_name: The name of the Runtime cloud cluster to which the Cloud Attachment is assigned. This field is populated only for Cloud Attachments., defaults to None
    :type cloud_molecule_name: str, optional
    :param cloud_name: The name of the associated Runtime cloud. This field is populated only for Cloud Attachments and Cloud runtime clusters., defaults to None
    :type cloud_name: str, optional
    :param cloud_owner_name: The account name of the associated Runtime cloud’s owner., defaults to None
    :type cloud_owner_name: str, optional
    :param cluster: cluster, defaults to None
    :type cluster: Nodes, optional
    :param created_by: The user ID (email address) of a user who created the Runtime., defaults to None
    :type created_by: str, optional
    :param current_version: A string that identifies the Runtime, Runtime cluster, or Runtime cloud's latest build., defaults to None
    :type current_version: str, optional
    :param date_installed: The installation date and time of the Runtime, Runtime cluster, or Runtime cloud. For Runtimes attached to a Runtime cloud, this is the installation date and time of the Runtime cloud., defaults to None
    :type date_installed: str, optional
    :param force_restart_time: The length of time, in milliseconds, that the platform waits before it forces the Runtime \(or Runtime cluster or Runtime cloud\) to restart after changes to the Atom’s configuration. Changes do not take effect until a restart occurs. However, regardless of this value, in a Runtime cluster or Runtime cloud that uses forked execution, automatic restart is deferred until currently running processes are complete., defaults to None
    :type force_restart_time: int, optional
    :param host_name: The name or IP address of the installation machine for the Runtime, Runtime cluster, or Runtime cloud. If the Runtime is attached to a Runtime cloud, the name of the Runtime cloud appears., defaults to None
    :type host_name: str, optional
    :param id_: A unique ID for the Runtime, Runtime cluster, or Runtime cloud., defaults to None
    :type id_: str, optional
    :param instance_id: \(For Runtimes attached to Runtime clouds\) A unique ID for the Runtime. The ID consists of the owning account ID followed by a period and a suffix., defaults to None
    :type instance_id: str, optional
    :param is_cloud_attachment: Indicates if a Runtime is a Cloud Attachment. It is set to `True` for the Cloud Attachment; otherwise, it is False for other Runtime. It is only populated for Cloud Attachments and Cloud runtime clusters., defaults to None
    :type is_cloud_attachment: bool, optional
    :param name: A user-defined name for the Runtime, Runtime cluster, or Runtime cloud., defaults to None
    :type name: str, optional
    :param purge_history_days: The number of days after a process run when the purging of logs, processed documents, and temporary data occurs. The default is 30 days. The maximum is 9999. A value of 0 disables purging., defaults to None
    :type purge_history_days: int, optional
    :param purge_immediate: If true, purges processed documents and temporary data immediately after a process ends. If you set this to true, also set purgeHistoryDays to a value greater than 0, such as 1. This combination not only purges your data right away, but also runs an extra cleanup process on a daily basis., defaults to None
    :type purge_immediate: bool, optional
    :param status: The status of the Runtime. Possible values are: UNKNOWN, ONLINE, WARNING, OFFLINE, defaults to None
    :type status: AtomStatus, optional
    :param type_: The type of Runtime. Possible values are: Cloud, Molecule, Atom, defaults to None
    :type type_: AtomType, optional
    :param status_detail: Provides more granular status details for the runtime. \<details\> \<summary\>Possible values:\</summary\>   When the runtime status is ONLINE:   - ONLINE_RUNNING   - ONLINE_STOPPING   - ONLINE_STARTING   - CLUSTER_ISSUE   - RESTARTING   - RESTARTING_TOO_LONG   - RESTARTING_DEAD   When the runtime status is OFFLINE:   - OFFLINE   - OFFLINE_NOT_STOPPED   - OFFLINE_NOT_INITIALIZED   - OFFLINE_WITH_ERROR   - OFFLINE_WITH_CLUSTER_ISSUE   - DELETED   - OFFBOARDING   When the runtime status is WARNING: CLUSTER_ISSUE   When the runtime status is UNKNOWN: UNKNOWN   \</details\>, defaults to None
    :type status_detail: str, optional
    """

    def __init__(
        self,
        capabilities: List[Capabilities] = SENTINEL,
        cloud_id: str = SENTINEL,
        cloud_molecule_id: str = SENTINEL,
        cloud_molecule_name: str = SENTINEL,
        cloud_name: str = SENTINEL,
        cloud_owner_name: str = SENTINEL,
        cluster: Nodes = SENTINEL,
        created_by: str = SENTINEL,
        current_version: str = SENTINEL,
        date_installed: str = SENTINEL,
        force_restart_time: int = SENTINEL,
        host_name: str = SENTINEL,
        id_: str = SENTINEL,
        instance_id: str = SENTINEL,
        is_cloud_attachment: bool = SENTINEL,
        name: str = SENTINEL,
        purge_history_days: int = SENTINEL,
        purge_immediate: bool = SENTINEL,
        status: AtomStatus = SENTINEL,
        type_: AtomType = SENTINEL,
        status_detail: str = SENTINEL,
        **kwargs,
    ):
        """Atom

        :param capabilities: capabilities, defaults to None
        :type capabilities: List[Capabilities], optional
        :param cloud_id: \(For Runtimes attached to Runtime clouds\) A unique ID assigned by the system to the Runtime cloud., defaults to None
        :type cloud_id: str, optional
        :param cloud_molecule_id: ID of the Runtime cloud cluster to which the Cloud Attachment is assigned. This field is populated only for Cloud Attachments., defaults to None
        :type cloud_molecule_id: str, optional
        :param cloud_molecule_name: The name of the Runtime cloud cluster to which the Cloud Attachment is assigned. This field is populated only for Cloud Attachments., defaults to None
        :type cloud_molecule_name: str, optional
        :param cloud_name: The name of the associated Runtime cloud. This field is populated only for Cloud Attachments and Cloud runtime clusters., defaults to None
        :type cloud_name: str, optional
        :param cloud_owner_name: The account name of the associated Runtime cloud’s owner., defaults to None
        :type cloud_owner_name: str, optional
        :param cluster: cluster, defaults to None
        :type cluster: Nodes, optional
        :param created_by: The user ID (email address) of a user who created the Runtime., defaults to None
        :type created_by: str, optional
        :param current_version: A string that identifies the Runtime, Runtime cluster, or Runtime cloud's latest build., defaults to None
        :type current_version: str, optional
        :param date_installed: The installation date and time of the Runtime, Runtime cluster, or Runtime cloud. For Runtimes attached to a Runtime cloud, this is the installation date and time of the Runtime cloud., defaults to None
        :type date_installed: str, optional
        :param force_restart_time: The length of time, in milliseconds, that the platform waits before it forces the Runtime \(or Runtime cluster or Runtime cloud\) to restart after changes to the Atom’s configuration. Changes do not take effect until a restart occurs. However, regardless of this value, in a Runtime cluster or Runtime cloud that uses forked execution, automatic restart is deferred until currently running processes are complete., defaults to None
        :type force_restart_time: int, optional
        :param host_name: The name or IP address of the installation machine for the Runtime, Runtime cluster, or Runtime cloud. If the Runtime is attached to a Runtime cloud, the name of the Runtime cloud appears., defaults to None
        :type host_name: str, optional
        :param id_: A unique ID for the Runtime, Runtime cluster, or Runtime cloud., defaults to None
        :type id_: str, optional
        :param instance_id: \(For Runtimes attached to Runtime clouds\) A unique ID for the Runtime. The ID consists of the owning account ID followed by a period and a suffix., defaults to None
        :type instance_id: str, optional
        :param is_cloud_attachment: Indicates if a Runtime is a Cloud Attachment. It is set to `True` for the Cloud Attachment; otherwise, it is False for other Runtime. It is only populated for Cloud Attachments and Cloud runtime clusters., defaults to None
        :type is_cloud_attachment: bool, optional
        :param name: A user-defined name for the Runtime, Runtime cluster, or Runtime cloud., defaults to None
        :type name: str, optional
        :param purge_history_days: The number of days after a process run when the purging of logs, processed documents, and temporary data occurs. The default is 30 days. The maximum is 9999. A value of 0 disables purging., defaults to None
        :type purge_history_days: int, optional
        :param purge_immediate: If true, purges processed documents and temporary data immediately after a process ends. If you set this to true, also set purgeHistoryDays to a value greater than 0, such as 1. This combination not only purges your data right away, but also runs an extra cleanup process on a daily basis., defaults to None
        :type purge_immediate: bool, optional
        :param status: The status of the Runtime. Possible values are: UNKNOWN, ONLINE, WARNING, OFFLINE, defaults to None
        :type status: AtomStatus, optional
        :param type_: The type of Runtime. Possible values are: Cloud, Molecule, Atom, defaults to None
        :type type_: AtomType, optional
        :param status_detail: Provides more granular status details for the runtime. \<details\> \<summary\>Possible values:\</summary\>   When the runtime status is ONLINE:   - ONLINE_RUNNING   - ONLINE_STOPPING   - ONLINE_STARTING   - CLUSTER_ISSUE   - RESTARTING   - RESTARTING_TOO_LONG   - RESTARTING_DEAD   When the runtime status is OFFLINE:   - OFFLINE   - OFFLINE_NOT_STOPPED   - OFFLINE_NOT_INITIALIZED   - OFFLINE_WITH_ERROR   - OFFLINE_WITH_CLUSTER_ISSUE   - DELETED   - OFFBOARDING   When the runtime status is WARNING: CLUSTER_ISSUE   When the runtime status is UNKNOWN: UNKNOWN   \</details\>, defaults to None
        :type status_detail: str, optional
        """
        if capabilities is not SENTINEL:
            self.capabilities = self._define_list(capabilities, Capabilities)
        if cloud_id is not SENTINEL:
            self.cloud_id = cloud_id
        if cloud_molecule_id is not SENTINEL:
            self.cloud_molecule_id = cloud_molecule_id
        if cloud_molecule_name is not SENTINEL:
            self.cloud_molecule_name = cloud_molecule_name
        if cloud_name is not SENTINEL:
            self.cloud_name = cloud_name
        if cloud_owner_name is not SENTINEL:
            self.cloud_owner_name = cloud_owner_name
        if cluster is not SENTINEL:
            self.cluster = self._define_object(cluster, Nodes)
        if created_by is not SENTINEL:
            self.created_by = created_by
        if current_version is not SENTINEL:
            self.current_version = current_version
        if date_installed is not SENTINEL:
            self.date_installed = date_installed
        if force_restart_time is not SENTINEL:
            self.force_restart_time = force_restart_time
        if host_name is not SENTINEL:
            self.host_name = host_name
        if id_ is not SENTINEL:
            self.id_ = id_
        if instance_id is not SENTINEL:
            self.instance_id = instance_id
        if is_cloud_attachment is not SENTINEL:
            self.is_cloud_attachment = is_cloud_attachment
        if name is not SENTINEL:
            self.name = name
        if purge_history_days is not SENTINEL:
            self.purge_history_days = purge_history_days
        if purge_immediate is not SENTINEL:
            self.purge_immediate = purge_immediate
        if status is not SENTINEL:
            self.status = self._enum_matching(status, AtomStatus.list(), "status")
        if type_ is not SENTINEL:
            self.type_ = self._enum_matching(type_, AtomType.list(), "type_")
        if status_detail is not SENTINEL:
            self.status_detail = status_detail
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    DeployedPackage,
    DeployedPackageBulkRequest,
    DeployedPackageBulkResponse,
    DeployedPackageQueryConfig,
    DeployedPackageQueryResponse,
)


class DeployedPackageService(BaseService):

    @cast_models
    def create_deployed_package(
        self, request_body: DeployedPackage = None
    ) -> Union[DeployedPackage, str]:
        """You can use the CREATE operation in two ways:
         - With `environmentId` and `packageId`, CREATE deploys the specified packaged component to the identified environment.
         - With `environmentId` and `componentId`, CREATE packages with the specified component and deploys the package to the specified environment.
         >**Note:** By default, deployment of listener processes are in a running state. To deploy a packaged listener process in a paused state, include the `listenerStatus` field with a value of `PAUSED`.

        :param request_body: The request body., defaults to None
        :type request_body: DeployedPackage, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedPackage, str]
        """

        Validator(DeployedPackage).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedPackage._unmap(response)
        if content == "application/xml":
            return DeployedPackage._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_deployed_package(self, id_: str) -> Union[DeployedPackage, str]:
        """Returns a single Deployed Package object based on the deployment ID.

        :param id_: The Deployed Package object you are attempting to DELETE.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedPackage, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedPackage._unmap(response)
        if content == "application/xml":
            return DeployedPackage._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_deployed_package(self, id_: str) -> None:
        """Removes the packaged component from the environment each with a specific IDs.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_deployed_package(
        self, request_body: DeployedPackageBulkRequest = None
    ) -> Union[DeployedPackageBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: DeployedPackageBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedPackageBulkResponse, str]
        """

        Validator(DeployedPackageBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedPackageBulkResponse._unmap(response)
        if content == "application/xml":
            return DeployedPackageBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_deployed_package(
        self, request_body: DeployedPackageQueryConfig = None
    ) -> Union[DeployedPackageQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: DeployedPackageQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedPackageQueryResponse, str]
        """

        Validator(DeployedPackageQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedPackageQueryResponse._unmap(response)
        if content == "application/xml":
            return DeployedPackageQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_deployed_package(
        self, request_body: str
    ) -> Union[DeployedPackageQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedPackageQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedPackage/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedPackageQueryResponse._unmap(response)
        if content == "application/xml":
            return DeployedPackageQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Deployment,
    DeploymentBulkRequest,
    DeploymentBulkResponse,
    DeploymentQueryConfig,
    DeploymentQueryResponse,
    ProcessEnvironmentAttachmentQueryConfig,
    ProcessEnvironmentAttachmentQueryResponse,
)


class DeploymentService(BaseService):

    @cast_models
    def create_deployment(
        self, request_body: Deployment = None
    ) -> Union[Deployment, str]:
        """The Deployment object is a deprecated API and should no longer be used. Boomi recommends that you take advantage of the API functionality provided by the [Packaged Component](https://help.boomi.com/docs/Atomsphere/Integration/AtomSphere%20API/r-atm-Packaged_Component_object_66fa92c8-948f-46c6-a521-3927ab431c84) and [Deployed Package objects](https://help.boomi.com/docs/Atomsphere/Integration/AtomSphere%20API/r-atm-Deployed_Package_object_897b5068-6daa-44e4-bf04-7e25385157a8) instead.

        :param request_body: The request body., defaults to None
        :type request_body: Deployment, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Deployment, str]
        """

        Validator(Deployment).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Deployment",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Deployment._unmap(response)
        if content == "application/xml":
            return Deployment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_deployment(self, id_: str) -> Union[Deployment, str]:
        """The Deployment object is a deprecated API and should no longer be used. Boomi recommends that you take advantage of the API functionality provided by the [Packaged Component](https://help.boomi.com/docs/Atomsphere/Integration/AtomSphere%20API/r-atm-Packaged_Component_object_66fa92c8-948f-46c6-a521-3927ab431c84) and [Deployed Package objects](https://help.boomi.com/docs/Atomsphere/Integration/AtomSphere%20API/r-atm-Deployed_Package_object_897b5068-6daa-44e4-bf04-7e25385157a8) instead.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Deployment, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Deployment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Deployment._unmap(response)
        if content == "application/xml":
            return Deployment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_deployment(
        self, request_body: DeploymentBulkRequest = None
    ) -> Union[DeploymentBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: DeploymentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeploymentBulkResponse, str]
        """

        Validator(DeploymentBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Deployment/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeploymentBulkResponse._unmap(response)
        if content == "application/xml":
            return DeploymentBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_deployment(
        self, request_body: DeploymentQueryConfig = None
    ) -> Union[DeploymentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: DeploymentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeploymentQueryResponse, str]
        """

        Validator(DeploymentQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Deployment/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeploymentQueryResponse._unmap(response)
        if content == "application/xml":
            return DeploymentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_deployment(
        self, request_body: str
    ) -> Union[DeploymentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeploymentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Deployment/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeploymentQueryResponse._unmap(response)
        if content == "application/xml":
            return DeploymentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_process_environment_attachment(
        self, request_body: ProcessEnvironmentAttachmentQueryConfig = None
    ) -> Union[ProcessEnvironmentAttachmentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: ProcessEnvironmentAttachmentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessEnvironmentAttachmentQueryResponse, str]
        """

        Validator(ProcessEnvironmentAttachmentQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessEnvironmentAttachment/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessEnvironmentAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return ProcessEnvironmentAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_process_environment_attachment(
        self, request_body: str
    ) -> Union[ProcessEnvironmentAttachmentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ProcessEnvironmentAttachmentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessEnvironmentAttachment/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ProcessEnvironmentAttachmentQueryResponse._unmap(response)
        if content == "application/xml":
            return ProcessEnvironmentAttachmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_process_environment_attachment(self, id_: str) -> None:
        """Detaches a process from an environment where the attachment is specified by the conceptual Process Environment Attachment object ID.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ProcessEnvironmentAttachment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import ReleaseIntegrationPack


class ReleaseIntegrationPackService(BaseService):

    @cast_models
    def create_release_integration_pack(
        self, request_body: ReleaseIntegrationPack = None
    ) -> Union[ReleaseIntegrationPack, str]:
        """Creates an immediate or scheduled release for a publisher integration pack.

        To schedule the publisher integration pack for release, you must specify the release schedule (immediate or scheduled).
        The `releaseOnDate` field is required if you schedule the release for a future date.

        :param request_body: The request body., defaults to None
        :type request_body: ReleaseIntegrationPack, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ReleaseIntegrationPack, str]
        """

        Validator(ReleaseIntegrationPack).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ReleaseIntegrationPack",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ReleaseIntegrationPack._unmap(response)
        if content == "application/xml":
            return ReleaseIntegrationPack._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_release_integration_pack(
        self, id_: str, request_body: ReleaseIntegrationPack = None
    ) -> Union[ReleaseIntegrationPack, str]:
        """Modifies the scheduled release of a publisher integration pack.

         > **Note:** The Update operation is only performed when there is an existing scheduled release request for the integration pack.

        :param request_body: The request body., defaults to None
        :type request_body: ReleaseIntegrationPack, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ReleaseIntegrationPack, str]
        """

        Validator(ReleaseIntegrationPack).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ReleaseIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ReleaseIntegrationPack._unmap(response)
        if content == "application/xml":
            return ReleaseIntegrationPack._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class AttachmentContentType(Enum):
    """An enumeration representing different categories.

    :cvar APPLICATIONXML: "application/xml"
    :vartype APPLICATIONXML: str
    :cvar APPLICATIONPDF: "application/pdf"
    :vartype APPLICATIONPDF: str
    :cvar APPLICATIONMSWORD: "application/msword"
    :vartype APPLICATIONMSWORD: str
    :cvar IMAGETIFF: "image/tiff"
    :vartype IMAGETIFF: str
    :cvar IMAGEJPEG: "image/jpeg"
    :vartype IMAGEJPEG: str
    :cvar TEXTPLAIN: "text/plain"
    :vartype TEXTPLAIN: str
    """

    APPLICATIONXML = "application/xml"
    APPLICATIONPDF = "application/pdf"
    APPLICATIONMSWORD = "application/msword"
    IMAGETIFF = "image/tiff"
    IMAGEJPEG = "image/jpeg"
    TEXTPLAIN = "text/plain"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AttachmentContentType._member_map_.values()))


@JsonMap(
    {
        "attachment_cache": "attachmentCache",
        "attachment_content_type": "attachmentContentType",
        "multiple_attachments": "multipleAttachments",
    }
)
class AttachmentInfo(BaseModel):
    """AttachmentInfo

    :param attachment_cache: attachment_cache, defaults to None
    :type attachment_cache: str, optional
    :param attachment_content_type: attachment_content_type, defaults to None
    :type attachment_content_type: List[AttachmentContentType], optional
    :param multiple_attachments: multiple_attachments, defaults to None
    :type multiple_attachments: bool, optional
    """

    def __init__(
        self,
        attachment_cache: str = SENTINEL,
        attachment_content_type: List[AttachmentContentType] = SENTINEL,
        multiple_attachments: bool = SENTINEL,
        **kwargs
    ):
        """AttachmentInfo

        :param attachment_cache: attachment_cache, defaults to None
        :type attachment_cache: str, optional
        :param attachment_content_type: attachment_content_type, defaults to None
        :type attachment_content_type: List[AttachmentContentType], optional
        :param multiple_attachments: multiple_attachments, defaults to None
        :type multiple_attachments: bool, optional
        """
        if attachment_cache is not SENTINEL:
            self.attachment_cache = attachment_cache
        if attachment_content_type is not SENTINEL:
            self.attachment_content_type = self._define_list(
                attachment_content_type, AttachmentContentType
            )
        if multiple_attachments is not SENTINEL:
            self.multiple_attachments = multiple_attachments
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ApplicationIdQual(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTIDQUALNA: "EDIFACTIDQUAL_NA"
    :vartype EDIFACTIDQUALNA: str
    :cvar EDIFACTIDQUAL1: "EDIFACTIDQUAL_1"
    :vartype EDIFACTIDQUAL1: str
    :cvar EDIFACTIDQUAL4: "EDIFACTIDQUAL_4"
    :vartype EDIFACTIDQUAL4: str
    :cvar EDIFACTIDQUAL5: "EDIFACTIDQUAL_5"
    :vartype EDIFACTIDQUAL5: str
    :cvar EDIFACTIDQUAL8: "EDIFACTIDQUAL_8"
    :vartype EDIFACTIDQUAL8: str
    :cvar EDIFACTIDQUAL9: "EDIFACTIDQUAL_9"
    :vartype EDIFACTIDQUAL9: str
    :cvar EDIFACTIDQUAL12: "EDIFACTIDQUAL_12"
    :vartype EDIFACTIDQUAL12: str
    :cvar EDIFACTIDQUAL14: "EDIFACTIDQUAL_14"
    :vartype EDIFACTIDQUAL14: str
    :cvar EDIFACTIDQUAL18: "EDIFACTIDQUAL_18"
    :vartype EDIFACTIDQUAL18: str
    :cvar EDIFACTIDQUAL22: "EDIFACTIDQUAL_22"
    :vartype EDIFACTIDQUAL22: str
    :cvar EDIFACTIDQUAL30: "EDIFACTIDQUAL_30"
    :vartype EDIFACTIDQUAL30: str
    :cvar EDIFACTIDQUAL31: "EDIFACTIDQUAL_31"
    :vartype EDIFACTIDQUAL31: str
    :cvar EDIFACTIDQUAL33: "EDIFACTIDQUAL_33"
    :vartype EDIFACTIDQUAL33: str
    :cvar EDIFACTIDQUAL34: "EDIFACTIDQUAL_34"
    :vartype EDIFACTIDQUAL34: str
    :cvar EDIFACTIDQUAL51: "EDIFACTIDQUAL_51"
    :vartype EDIFACTIDQUAL51: str
    :cvar EDIFACTIDQUAL52: "EDIFACTIDQUAL_52"
    :vartype EDIFACTIDQUAL52: str
    :cvar EDIFACTIDQUAL53: "EDIFACTIDQUAL_53"
    :vartype EDIFACTIDQUAL53: str
    :cvar EDIFACTIDQUAL54: "EDIFACTIDQUAL_54"
    :vartype EDIFACTIDQUAL54: str
    :cvar EDIFACTIDQUAL55: "EDIFACTIDQUAL_55"
    :vartype EDIFACTIDQUAL55: str
    :cvar EDIFACTIDQUAL57: "EDIFACTIDQUAL_57"
    :vartype EDIFACTIDQUAL57: str
    :cvar EDIFACTIDQUAL58: "EDIFACTIDQUAL_58"
    :vartype EDIFACTIDQUAL58: str
    :cvar EDIFACTIDQUAL59: "EDIFACTIDQUAL_59"
    :vartype EDIFACTIDQUAL59: str
    :cvar EDIFACTIDQUAL61: "EDIFACTIDQUAL_61"
    :vartype EDIFACTIDQUAL61: str
    :cvar EDIFACTIDQUAL63: "EDIFACTIDQUAL_63"
    :vartype EDIFACTIDQUAL63: str
    :cvar EDIFACTIDQUAL65: "EDIFACTIDQUAL_65"
    :vartype EDIFACTIDQUAL65: str
    :cvar EDIFACTIDQUAL80: "EDIFACTIDQUAL_80"
    :vartype EDIFACTIDQUAL80: str
    :cvar EDIFACTIDQUAL82: "EDIFACTIDQUAL_82"
    :vartype EDIFACTIDQUAL82: str
    :cvar EDIFACTIDQUAL84: "EDIFACTIDQUAL_84"
    :vartype EDIFACTIDQUAL84: str
    :cvar EDIFACTIDQUAL85: "EDIFACTIDQUAL_85"
    :vartype EDIFACTIDQUAL85: str
    :cvar EDIFACTIDQUAL86: "EDIFACTIDQUAL_86"
    :vartype EDIFACTIDQUAL86: str
    :cvar EDIFACTIDQUAL87: "EDIFACTIDQUAL_87"
    :vartype EDIFACTIDQUAL87: str
    :cvar EDIFACTIDQUAL89: "EDIFACTIDQUAL_89"
    :vartype EDIFACTIDQUAL89: str
    :cvar EDIFACTIDQUAL90: "EDIFACTIDQUAL_90"
    :vartype EDIFACTIDQUAL90: str
    :cvar EDIFACTIDQUAL91: "EDIFACTIDQUAL_91"
    :vartype EDIFACTIDQUAL91: str
    :cvar EDIFACTIDQUAL92: "EDIFACTIDQUAL_92"
    :vartype EDIFACTIDQUAL92: str
    :cvar EDIFACTIDQUAL103: "EDIFACTIDQUAL_103"
    :vartype EDIFACTIDQUAL103: str
    :cvar EDIFACTIDQUAL128: "EDIFACTIDQUAL_128"
    :vartype EDIFACTIDQUAL128: str
    :cvar EDIFACTIDQUAL129: "EDIFACTIDQUAL_129"
    :vartype EDIFACTIDQUAL129: str
    :cvar EDIFACTIDQUAL144: "EDIFACTIDQUAL_144"
    :vartype EDIFACTIDQUAL144: str
    :cvar EDIFACTIDQUAL145: "EDIFACTIDQUAL_145"
    :vartype EDIFACTIDQUAL145: str
    :cvar EDIFACTIDQUAL146: "EDIFACTIDQUAL_146"
    :vartype EDIFACTIDQUAL146: str
    :cvar EDIFACTIDQUAL147: "EDIFACTIDQUAL_147"
    :vartype EDIFACTIDQUAL147: str
    :cvar EDIFACTIDQUAL148: "EDIFACTIDQUAL_148"
    :vartype EDIFACTIDQUAL148: str
    :cvar EDIFACTIDQUALZ01: "EDIFACTIDQUAL_Z01"
    :vartype EDIFACTIDQUALZ01: str
    :cvar EDIFACTIDQUALZZZ: "EDIFACTIDQUAL_ZZZ"
    :vartype EDIFACTIDQUALZZZ: str
    :cvar EDIFACTIDQUALZZ: "EDIFACTIDQUAL_ZZ"
    :vartype EDIFACTIDQUALZZ: str
    """

    EDIFACTIDQUALNA = "EDIFACTIDQUAL_NA"
    EDIFACTIDQUAL1 = "EDIFACTIDQUAL_1"
    EDIFACTIDQUAL4 = "EDIFACTIDQUAL_4"
    EDIFACTIDQUAL5 = "EDIFACTIDQUAL_5"
    EDIFACTIDQUAL8 = "EDIFACTIDQUAL_8"
    EDIFACTIDQUAL9 = "EDIFACTIDQUAL_9"
    EDIFACTIDQUAL12 = "EDIFACTIDQUAL_12"
    EDIFACTIDQUAL14 = "EDIFACTIDQUAL_14"
    EDIFACTIDQUAL18 = "EDIFACTIDQUAL_18"
    EDIFACTIDQUAL22 = "EDIFACTIDQUAL_22"
    EDIFACTIDQUAL30 = "EDIFACTIDQUAL_30"
    EDIFACTIDQUAL31 = "EDIFACTIDQUAL_31"
    EDIFACTIDQUAL33 = "EDIFACTIDQUAL_33"
    EDIFACTIDQUAL34 = "EDIFACTIDQUAL_34"
    EDIFACTIDQUAL51 = "EDIFACTIDQUAL_51"
    EDIFACTIDQUAL52 = "EDIFACTIDQUAL_52"
    EDIFACTIDQUAL53 = "EDIFACTIDQUAL_53"
    EDIFACTIDQUAL54 = "EDIFACTIDQUAL_54"
    EDIFACTIDQUAL55 = "EDIFACTIDQUAL_55"
    EDIFACTIDQUAL57 = "EDIFACTIDQUAL_57"
    EDIFACTIDQUAL58 = "EDIFACTIDQUAL_58"
    EDIFACTIDQUAL59 = "EDIFACTIDQUAL_59"
    EDIFACTIDQUAL61 = "EDIFACTIDQUAL_61"
    EDIFACTIDQUAL63 = "EDIFACTIDQUAL_63"
    EDIFACTIDQUAL65 = "EDIFACTIDQUAL_65"
    EDIFACTIDQUAL80 = "EDIFACTIDQUAL_80"
    EDIFACTIDQUAL82 = "EDIFACTIDQUAL_82"
    EDIFACTIDQUAL84 = "EDIFACTIDQUAL_84"
    EDIFACTIDQUAL85 = "EDIFACTIDQUAL_85"
    EDIFACTIDQUAL86 = "EDIFACTIDQUAL_86"
    EDIFACTIDQUAL87 = "EDIFACTIDQUAL_87"
    EDIFACTIDQUAL89 = "EDIFACTIDQUAL_89"
    EDIFACTIDQUAL90 = "EDIFACTIDQUAL_90"
    EDIFACTIDQUAL91 = "EDIFACTIDQUAL_91"
    EDIFACTIDQUAL92 = "EDIFACTIDQUAL_92"
    EDIFACTIDQUAL103 = "EDIFACTIDQUAL_103"
    EDIFACTIDQUAL128 = "EDIFACTIDQUAL_128"
    EDIFACTIDQUAL129 = "EDIFACTIDQUAL_129"
    EDIFACTIDQUAL144 = "EDIFACTIDQUAL_144"
    EDIFACTIDQUAL145 = "EDIFACTIDQUAL_145"
    EDIFACTIDQUAL146 = "EDIFACTIDQUAL_146"
    EDIFACTIDQUAL147 = "EDIFACTIDQUAL_147"
    EDIFACTIDQUAL148 = "EDIFACTIDQUAL_148"
    EDIFACTIDQUALZ01 = "EDIFACTIDQUAL_Z01"
    EDIFACTIDQUALZZZ = "EDIFACTIDQUAL_ZZZ"
    EDIFACTIDQUALZZ = "EDIFACTIDQUAL_ZZ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ApplicationIdQual._member_map_.values()))


@JsonMap(
    {
        "application_id": "applicationId",
        "application_id_qual": "applicationIdQual",
        "use_functional_groups": "useFunctionalGroups",
    }
)
class UngControlInfo(BaseModel):
    """UngControlInfo

    :param application_id: application_id, defaults to None
    :type application_id: str, optional
    :param application_id_qual: application_id_qual, defaults to None
    :type application_id_qual: ApplicationIdQual, optional
    :param use_functional_groups: use_functional_groups, defaults to None
    :type use_functional_groups: bool, optional
    """

    def __init__(
        self,
        application_id: str = SENTINEL,
        application_id_qual: ApplicationIdQual = SENTINEL,
        use_functional_groups: bool = SENTINEL,
        **kwargs
    ):
        """UngControlInfo

        :param application_id: application_id, defaults to None
        :type application_id: str, optional
        :param application_id_qual: application_id_qual, defaults to None
        :type application_id_qual: ApplicationIdQual, optional
        :param use_functional_groups: use_functional_groups, defaults to None
        :type use_functional_groups: bool, optional
        """
        if application_id is not SENTINEL:
            self.application_id = application_id
        if application_id_qual is not SENTINEL:
            self.application_id_qual = self._enum_matching(
                application_id_qual, ApplicationIdQual.list(), "application_id_qual"
            )
        if use_functional_groups is not SENTINEL:
            self.use_functional_groups = use_functional_groups
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_group_user_role import AccountGroupUserRoleService
from ...models import (
    AccountGroupUserRole,
    AccountGroupUserRoleQueryResponse,
    AccountGroupUserRoleQueryConfig,
)


class AccountGroupUserRoleServiceAsync(AccountGroupUserRoleService):
    """
    Async Wrapper for AccountGroupUserRoleServiceAsync
    """

    def create_account_group_user_role(
        self, request_body: AccountGroupUserRole = None
    ) -> Awaitable[Union[AccountGroupUserRole, str]]:
        return to_async(super().create_account_group_user_role)(request_body)

    def query_account_group_user_role(
        self, request_body: AccountGroupUserRoleQueryConfig = None
    ) -> Awaitable[Union[AccountGroupUserRoleQueryResponse, str]]:
        return to_async(super().query_account_group_user_role)(request_body)

    def query_more_account_group_user_role(
        self, request_body: str
    ) -> Awaitable[Union[AccountGroupUserRoleQueryResponse, str]]:
        return to_async(super().query_more_account_group_user_role)(request_body)

    def delete_account_group_user_role(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_account_group_user_role)(id_)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..connection_licensing_report import ConnectionLicensingReportService
from ...models import ConnectionLicensingDownload, ConnectionLicensingReport


class ConnectionLicensingReportServiceAsync(ConnectionLicensingReportService):
    """
    Async Wrapper for ConnectionLicensingReportServiceAsync
    """

    def create_connection_licensing_report(
        self, request_body: ConnectionLicensingReport = None
    ) -> Awaitable[Union[ConnectionLicensingDownload, str]]:
        return to_async(super().create_connection_licensing_report)(request_body)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .listener_status_simple_expression import ListenerStatusSimpleExpression
from .listener_status_grouping_expression import ListenerStatusGroupingExpression


class ListenerStatusExpressionGuard(OneOfBaseModel):
    class_list = {
        "ListenerStatusSimpleExpression": ListenerStatusSimpleExpression,
        "ListenerStatusGroupingExpression": ListenerStatusGroupingExpression,
    }


ListenerStatusExpression = Union[
    ListenerStatusSimpleExpression, ListenerStatusGroupingExpression
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import AtomPurge


class AtomPurgeService(BaseService):

    @cast_models
    def update_atom_purge(
        self, id_: str, request_body: AtomPurge = None
    ) -> Union[AtomPurge, str]:
        """You can use the Purge Runtime cloud attachment operation to programmatically start the purge process for test and browse components, logs, processed documents, and temporary data for a Runtime Cloud attachment.

        :param request_body: The request body., defaults to None
        :type request_body: AtomPurge, optional
        :param id_: The unique ID assigned by the system to the Runtime cloud attachment. The Runtime ID is found in the user interface by navigating to **Manage > Runtime Management** and viewing the Runtime Information panel for a selected Runtime.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomPurge, str]
        """

        Validator(AtomPurge).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AtomPurge/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomPurge._unmap(response)
        if content == "application/xml":
            return AtomPurge._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .odette_unb_control_info import OdetteUnbControlInfo
from .odette_unh_control_info import OdetteUnhControlInfo


@JsonMap(
    {
        "odette_unb_control_info": "OdetteUNBControlInfo",
        "odette_unh_control_info": "OdetteUNHControlInfo",
    }
)
class OdetteControlInfo(BaseModel):
    """OdetteControlInfo

    :param odette_unb_control_info: odette_unb_control_info
    :type odette_unb_control_info: OdetteUnbControlInfo
    :param odette_unh_control_info: odette_unh_control_info
    :type odette_unh_control_info: OdetteUnhControlInfo
    """

    def __init__(
        self,
        odette_unb_control_info: OdetteUnbControlInfo,
        odette_unh_control_info: OdetteUnhControlInfo,
        **kwargs,
    ):
        """OdetteControlInfo

        :param odette_unb_control_info: odette_unb_control_info
        :type odette_unb_control_info: OdetteUnbControlInfo
        :param odette_unh_control_info: odette_unh_control_info
        :type odette_unh_control_info: OdetteUnhControlInfo
        """
        self.odette_unb_control_info = self._define_object(
            odette_unb_control_info, OdetteUnbControlInfo
        )
        self.odette_unh_control_info = self._define_object(
            odette_unh_control_info, OdetteUnhControlInfo
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extensions_inputs import MapExtensionsInputs
from .map_extensions_function_mappings import MapExtensionsFunctionMappings
from .map_extensions_outputs import MapExtensionsOutputs
from .map_extensions_function_steps import MapExtensionsFunctionSteps


@JsonMap(
    {
        "inputs": "Inputs",
        "mappings": "Mappings",
        "outputs": "Outputs",
        "steps": "Steps",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "environment_map_extension_id": "environmentMapExtensionId",
        "id_": "id",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
    }
)
class EnvironmentMapExtensionUserDefinedFunction(BaseModel):
    """EnvironmentMapExtensionUserDefinedFunction

    :param inputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
    :type inputs: MapExtensionsInputs
    :param mappings: Defines the mapping of inputs and outputs for the user-defined function and each function step. It uses the following attributes:  1. fromFunction - represents the function ID from which you are mapping.  2. fromKey - represents the function's output key from which you are mapping.  3. toFunction - represents the function ID to which you are mapping.  4. toKey - represents the function's input key to which you are mapping.
    :type mappings: MapExtensionsFunctionMappings
    :param outputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. See the following row for more information. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
    :type outputs: MapExtensionsOutputs
    :param steps: Defines the individual function steps and the order in which they need to occur within the greater user-defined function. The following attributes are used: 1.`position` - represents the step's number order in the greater function. 2.`cacheType` - indicates the caching behavior of the individual function step. The allowed `cacheType` values are:1.`None` \(default, if omitted in the request\)— It does not use Map function caching. 2. `ByDocument` — Caches the map function’s input and output values for each processed document. 3. `ByMap` — Caches the map function’s input and output values for each processed map. -   `id` - represents the function step's ID in the format of "FUNCEXT--xxxxxxxxxx". 4. `type` - represents the type of function \(for example, "MathCeil" or "CustomScripting"\).\<br /\> Within the `Steps` element, you also need to define the following `input` and `output` variables for each function step:\<br /\>1. `default` - Optional. Specifies the input value that the function uses if not provided by the user.\<br /\>2. `name` - the user-defined name of the associated input or output.   \>**Note:** The user interface automatically uses the used function type as the step name, but you can use this API object to change function step names. 3. `key` - the number ID assigned to a function step. This key is used to map function steps together in the `Mappings` attribute.
    :type steps: MapExtensionsFunctionSteps
    :param created_by: The user ID of the user who created the user-defined function., defaults to None
    :type created_by: str, optional
    :param created_date: Timestamp of the creation of the user-defined function., defaults to None
    :type created_date: str, optional
    :param deleted: This variable indicates the deleted status of the user defined function component. If the value is true, it indicates the deletion of the referenced user-defined function. A false value indicates that the referenced user-defined function is not deleted and is available for use., defaults to None
    :type deleted: bool, optional
    :param description: Optional. Additional details about the user-defined function component., defaults to None
    :type description: str, optional
    :param environment_map_extension_id: The ID of an environment map extension. **Important:** This and other Environment Map Extension API objects require the client to know the ID of the environment map extension. In the user-defined function interface, click **Copy EME ID** to easily copy this ID for use in your API requests, or query the [Environment Map Extensions Summary object](/api/platformapi#tag/EnvironmentMapExtensionsSummary)., defaults to None
    :type environment_map_extension_id: str, optional
    :param id_: Required. Represents the unique, system-generated ID of the extended user-defined function., defaults to None
    :type id_: str, optional
    :param modified_by: The user ID of the user who last updated the user-defined function., defaults to None
    :type modified_by: str, optional
    :param modified_date: Timestamp of when the user-defined function was last updated., defaults to None
    :type modified_date: str, optional
    :param name: Required. Represents the name of the user-defined function component., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        inputs: MapExtensionsInputs,
        mappings: MapExtensionsFunctionMappings,
        outputs: MapExtensionsOutputs,
        steps: MapExtensionsFunctionSteps,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        deleted: bool = SENTINEL,
        description: str = SENTINEL,
        environment_map_extension_id: str = SENTINEL,
        id_: str = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        name: str = SENTINEL,
        **kwargs,
    ):
        """EnvironmentMapExtensionUserDefinedFunction

        :param inputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
        :type inputs: MapExtensionsInputs
        :param mappings: Defines the mapping of inputs and outputs for the user-defined function and each function step. It uses the following attributes:  1. fromFunction - represents the function ID from which you are mapping.  2. fromKey - represents the function's output key from which you are mapping.  3. toFunction - represents the function ID to which you are mapping.  4. toKey - represents the function's input key to which you are mapping.
        :type mappings: MapExtensionsFunctionMappings
        :param outputs: Lists the function's input and outputs according to their user-given names and keys. You must list inputs and outputs sequentially in order according to their key values. See the following row for more information. When creating or updating functions, it requires all input and output values in the request regardless if they are to be mapped or populated with a default value.   The maximum number of inputs or outputs is 100.
        :type outputs: MapExtensionsOutputs
        :param steps: Defines the individual function steps and the order in which they need to occur within the greater user-defined function. The following attributes are used: 1.`position` - represents the step's number order in the greater function. 2.`cacheType` - indicates the caching behavior of the individual function step. The allowed `cacheType` values are:1.`None` \(default, if omitted in the request\)— It does not use Map function caching. 2. `ByDocument` — Caches the map function’s input and output values for each processed document. 3. `ByMap` — Caches the map function’s input and output values for each processed map. -   `id` - represents the function step's ID in the format of "FUNCEXT--xxxxxxxxxx". 4. `type` - represents the type of function \(for example, "MathCeil" or "CustomScripting"\).\<br /\> Within the `Steps` element, you also need to define the following `input` and `output` variables for each function step:\<br /\>1. `default` - Optional. Specifies the input value that the function uses if not provided by the user.\<br /\>2. `name` - the user-defined name of the associated input or output.   \>**Note:** The user interface automatically uses the used function type as the step name, but you can use this API object to change function step names. 3. `key` - the number ID assigned to a function step. This key is used to map function steps together in the `Mappings` attribute.
        :type steps: MapExtensionsFunctionSteps
        :param created_by: The user ID of the user who created the user-defined function., defaults to None
        :type created_by: str, optional
        :param created_date: Timestamp of the creation of the user-defined function., defaults to None
        :type created_date: str, optional
        :param deleted: This variable indicates the deleted status of the user defined function component. If the value is true, it indicates the deletion of the referenced user-defined function. A false value indicates that the referenced user-defined function is not deleted and is available for use., defaults to None
        :type deleted: bool, optional
        :param description: Optional. Additional details about the user-defined function component., defaults to None
        :type description: str, optional
        :param environment_map_extension_id: The ID of an environment map extension. **Important:** This and other Environment Map Extension API objects require the client to know the ID of the environment map extension. In the user-defined function interface, click **Copy EME ID** to easily copy this ID for use in your API requests, or query the [Environment Map Extensions Summary object](/api/platformapi#tag/EnvironmentMapExtensionsSummary)., defaults to None
        :type environment_map_extension_id: str, optional
        :param id_: Required. Represents the unique, system-generated ID of the extended user-defined function., defaults to None
        :type id_: str, optional
        :param modified_by: The user ID of the user who last updated the user-defined function., defaults to None
        :type modified_by: str, optional
        :param modified_date: Timestamp of when the user-defined function was last updated., defaults to None
        :type modified_date: str, optional
        :param name: Required. Represents the name of the user-defined function component., defaults to None
        :type name: str, optional
        """
        self.inputs = self._define_object(inputs, MapExtensionsInputs)
        self.mappings = self._define_object(mappings, MapExtensionsFunctionMappings)
        self.outputs = self._define_object(outputs, MapExtensionsOutputs)
        self.steps = self._define_object(steps, MapExtensionsFunctionSteps)
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if deleted is not SENTINEL:
            self.deleted = deleted
        if description is not SENTINEL:
            self.description = description
        if environment_map_extension_id is not SENTINEL:
            self.environment_map_extension_id = environment_map_extension_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class SegmentTerminatorValue(Enum):
    """An enumeration representing different categories.

    :cvar NEWLINE: "newline"
    :vartype NEWLINE: str
    :cvar SINGLEQUOTE: "singlequote"
    :vartype SINGLEQUOTE: str
    :cvar TILDE: "tilde"
    :vartype TILDE: str
    :cvar CARRIAGERETURN: "carriagereturn"
    :vartype CARRIAGERETURN: str
    :cvar BYTECHARACTER: "bytecharacter"
    :vartype BYTECHARACTER: str
    :cvar OTHERCHARACTER: "othercharacter"
    :vartype OTHERCHARACTER: str
    """

    NEWLINE = "newline"
    SINGLEQUOTE = "singlequote"
    TILDE = "tilde"
    CARRIAGERETURN = "carriagereturn"
    BYTECHARACTER = "bytecharacter"
    OTHERCHARACTER = "othercharacter"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, SegmentTerminatorValue._member_map_.values())
        )


@JsonMap(
    {
        "segment_terminator_special": "segmentTerminatorSpecial",
        "segment_terminator_value": "segmentTerminatorValue",
    }
)
class EdiSegmentTerminator(BaseModel):
    """EdiSegmentTerminator

    :param segment_terminator_special: segment_terminator_special, defaults to None
    :type segment_terminator_special: str, optional
    :param segment_terminator_value: segment_terminator_value, defaults to None
    :type segment_terminator_value: SegmentTerminatorValue, optional
    """

    def __init__(
        self,
        segment_terminator_special: str = SENTINEL,
        segment_terminator_value: SegmentTerminatorValue = SENTINEL,
        **kwargs
    ):
        """EdiSegmentTerminator

        :param segment_terminator_special: segment_terminator_special, defaults to None
        :type segment_terminator_special: str, optional
        :param segment_terminator_value: segment_terminator_value, defaults to None
        :type segment_terminator_value: SegmentTerminatorValue, optional
        """
        if segment_terminator_special is not SENTINEL:
            self.segment_terminator_special = segment_terminator_special
        if segment_terminator_value is not SENTINEL:
            self.segment_terminator_value = self._enum_matching(
                segment_terminator_value,
                SegmentTerminatorValue.list(),
                "segment_terminator_value",
            )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .http_path_elements import HttpPathElements
from .http_reflect_headers import HttpReflectHeaders
from .http_request_headers import HttpRequestHeaders
from .http_response_header_mapping import HttpResponseHeaderMapping


class HttpGetOptionsMethodType(Enum):
    """An enumeration representing different categories.

    :cvar GET: "GET"
    :vartype GET: str
    :cvar POST: "POST"
    :vartype POST: str
    :cvar PUT: "PUT"
    :vartype PUT: str
    :cvar DELETE: "DELETE"
    :vartype DELETE: str
    """

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, HttpGetOptionsMethodType._member_map_.values())
        )


class HttpGetOptionsRequestProfileType(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar XML: "XML"
    :vartype XML: str
    :cvar JSON: "JSON"
    :vartype JSON: str
    """

    NONE = "NONE"
    XML = "XML"
    JSON = "JSON"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HttpGetOptionsRequestProfileType._member_map_.values(),
            )
        )


class HttpGetOptionsResponseProfileType(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar XML: "XML"
    :vartype XML: str
    :cvar JSON: "JSON"
    :vartype JSON: str
    """

    NONE = "NONE"
    XML = "XML"
    JSON = "JSON"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HttpGetOptionsResponseProfileType._member_map_.values(),
            )
        )


@JsonMap(
    {
        "data_content_type": "dataContentType",
        "follow_redirects": "followRedirects",
        "method_type": "methodType",
        "path_elements": "pathElements",
        "reflect_headers": "reflectHeaders",
        "request_headers": "requestHeaders",
        "request_profile": "requestProfile",
        "request_profile_type": "requestProfileType",
        "response_header_mapping": "responseHeaderMapping",
        "response_profile": "responseProfile",
        "response_profile_type": "responseProfileType",
        "return_errors": "returnErrors",
        "use_default_options": "useDefaultOptions",
    }
)
class HttpGetOptions(BaseModel):
    """HttpGetOptions

    :param data_content_type: data_content_type, defaults to None
    :type data_content_type: str, optional
    :param follow_redirects: follow_redirects, defaults to None
    :type follow_redirects: bool, optional
    :param method_type: method_type, defaults to None
    :type method_type: HttpGetOptionsMethodType, optional
    :param path_elements: path_elements
    :type path_elements: HttpPathElements
    :param reflect_headers: reflect_headers, defaults to None
    :type reflect_headers: HttpReflectHeaders, optional
    :param request_headers: request_headers
    :type request_headers: HttpRequestHeaders
    :param request_profile: request_profile, defaults to None
    :type request_profile: str, optional
    :param request_profile_type: request_profile_type, defaults to None
    :type request_profile_type: HttpGetOptionsRequestProfileType, optional
    :param response_header_mapping: response_header_mapping
    :type response_header_mapping: HttpResponseHeaderMapping
    :param response_profile: response_profile, defaults to None
    :type response_profile: str, optional
    :param response_profile_type: response_profile_type, defaults to None
    :type response_profile_type: HttpGetOptionsResponseProfileType, optional
    :param return_errors: return_errors, defaults to None
    :type return_errors: bool, optional
    :param use_default_options: use_default_options, defaults to None
    :type use_default_options: bool, optional
    """

    def __init__(
        self,
        path_elements: HttpPathElements,
        request_headers: HttpRequestHeaders,
        response_header_mapping: HttpResponseHeaderMapping,
        data_content_type: str = SENTINEL,
        follow_redirects: bool = SENTINEL,
        method_type: HttpGetOptionsMethodType = SENTINEL,
        reflect_headers: HttpReflectHeaders = SENTINEL,
        request_profile: str = SENTINEL,
        request_profile_type: HttpGetOptionsRequestProfileType = SENTINEL,
        response_profile: str = SENTINEL,
        response_profile_type: HttpGetOptionsResponseProfileType = SENTINEL,
        return_errors: bool = SENTINEL,
        use_default_options: bool = SENTINEL,
        **kwargs,
    ):
        """HttpGetOptions

        :param data_content_type: data_content_type, defaults to None
        :type data_content_type: str, optional
        :param follow_redirects: follow_redirects, defaults to None
        :type follow_redirects: bool, optional
        :param method_type: method_type, defaults to None
        :type method_type: HttpGetOptionsMethodType, optional
        :param path_elements: path_elements
        :type path_elements: HttpPathElements
        :param reflect_headers: reflect_headers, defaults to None
        :type reflect_headers: HttpReflectHeaders, optional
        :param request_headers: request_headers
        :type request_headers: HttpRequestHeaders
        :param request_profile: request_profile, defaults to None
        :type request_profile: str, optional
        :param request_profile_type: request_profile_type, defaults to None
        :type request_profile_type: HttpGetOptionsRequestProfileType, optional
        :param response_header_mapping: response_header_mapping
        :type response_header_mapping: HttpResponseHeaderMapping
        :param response_profile: response_profile, defaults to None
        :type response_profile: str, optional
        :param response_profile_type: response_profile_type, defaults to None
        :type response_profile_type: HttpGetOptionsResponseProfileType, optional
        :param return_errors: return_errors, defaults to None
        :type return_errors: bool, optional
        :param use_default_options: use_default_options, defaults to None
        :type use_default_options: bool, optional
        """
        if data_content_type is not SENTINEL:
            self.data_content_type = data_content_type
        if follow_redirects is not SENTINEL:
            self.follow_redirects = follow_redirects
        if method_type is not SENTINEL:
            self.method_type = self._enum_matching(
                method_type, HttpGetOptionsMethodType.list(), "method_type"
            )
        self.path_elements = self._define_object(path_elements, HttpPathElements)
        if reflect_headers is not SENTINEL:
            self.reflect_headers = self._define_object(
                reflect_headers, HttpReflectHeaders
            )
        self.request_headers = self._define_object(request_headers, HttpRequestHeaders)
        if request_profile is not SENTINEL:
            self.request_profile = request_profile
        if request_profile_type is not SENTINEL:
            self.request_profile_type = self._enum_matching(
                request_profile_type,
                HttpGetOptionsRequestProfileType.list(),
                "request_profile_type",
            )
        self.response_header_mapping = self._define_object(
            response_header_mapping, HttpResponseHeaderMapping
        )
        if response_profile is not SENTINEL:
            self.response_profile = response_profile
        if response_profile_type is not SENTINEL:
            self.response_profile_type = self._enum_matching(
                response_profile_type,
                HttpGetOptionsResponseProfileType.list(),
                "response_profile_type",
            )
        if return_errors is not SENTINEL:
            self.return_errors = return_errors
        if use_default_options is not SENTINEL:
            self.use_default_options = use_default_options
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Account,
    AccountBulkRequest,
    AccountBulkResponse,
    AccountQueryConfig,
    AccountQueryResponse,
)


class AccountService(BaseService):

    @cast_models
    def get_account(self, id_: str) -> Union[Account, str]:
        """Retrieves a single Account object using the supplied account ID. This GET operation returns details only for the parent account and does not include any accounts created by the parent account.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Account, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Account/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Account._unmap(response)
        if content == "application/xml":
            return Account._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_account(
        self, request_body: AccountBulkRequest = None
    ) -> Union[AccountBulkResponse, str]:
        """The bulk GET operation returns multiple Account objects based on the supplied account IDs, to a maximum of 100. To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AccountBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountBulkResponse, str]
        """

        Validator(AccountBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Account/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountBulkResponse._unmap(response)
        if content == "application/xml":
            return AccountBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_account(
        self, request_body: AccountQueryConfig = None
    ) -> Union[AccountQueryResponse, str]:
        """The Account Query API returns accounts that were created by the parent account identified in the request URL.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: AccountQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountQueryResponse, str]
        """

        Validator(AccountQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Account/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_account(self, request_body: str) -> Union[AccountQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Account/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"contact_name": "contactName"})
class ContactInfo(BaseModel):
    """ContactInfo

    :param address1: address1
    :type address1: str
    :param address2: address2
    :type address2: str
    :param city: city
    :type city: str
    :param contact_name: contact_name
    :type contact_name: str
    :param country: country
    :type country: str
    :param email: email
    :type email: str
    :param fax: fax
    :type fax: str
    :param phone: phone
    :type phone: str
    :param postalcode: postalcode
    :type postalcode: str
    :param state: state
    :type state: str
    """

    def __init__(
        self,
        address1: str,
        address2: str,
        city: str,
        contact_name: str,
        country: str,
        email: str,
        fax: str,
        phone: str,
        postalcode: str,
        state: str,
        **kwargs
    ):
        """ContactInfo

        :param address1: address1
        :type address1: str
        :param address2: address2
        :type address2: str
        :param city: city
        :type city: str
        :param contact_name: contact_name
        :type contact_name: str
        :param country: country
        :type country: str
        :param email: email
        :type email: str
        :param fax: fax
        :type fax: str
        :param phone: phone
        :type phone: str
        :param postalcode: postalcode
        :type postalcode: str
        :param state: state
        :type state: str
        """
        self.address1 = address1
        self.address2 = address2
        self.city = city
        self.contact_name = contact_name
        self.country = country
        self.email = email
        self.fax = fax
        self.phone = phone
        self.postalcode = postalcode
        self.state = state
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from xml.etree import ElementTree as ET
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .encrypted_values import EncryptedValues
import warnings


class ComponentType(Enum):
    """An enumeration representing different categories.

    :cvar CERTIFICATE: "certificate"
    :vartype CERTIFICATE: str
    :cvar CONNECTORACTION: "connector-action"
    :vartype CONNECTORACTION: str
    :cvar CONNECTORSETTINGS: "connector-settings"
    :vartype CONNECTORSETTINGS: str
    :cvar CROSSREF: "crossref"
    :vartype CROSSREF: str
    :cvar DOCUMENTCACHE: "documentcache"
    :vartype DOCUMENTCACHE: str
    :cvar TRANSFORMMAP: "transform.map"
    :vartype TRANSFORMMAP: str
    :cvar TRANSFORMFUNCTION: "transform.function"
    :vartype TRANSFORMFUNCTION: str
    :cvar CERTIFICATEPGP: "certificate.pgp"
    :vartype CERTIFICATEPGP: str
    :cvar PROCESS: "process"
    :vartype PROCESS: str
    :cvar PROCESSPROPERTY: "processproperty"
    :vartype PROCESSPROPERTY: str
    :cvar PROFILEDB: "profile.db"
    :vartype PROFILEDB: str
    :cvar PROFILEEDI: "profile.edi"
    :vartype PROFILEEDI: str
    :cvar PROFILEFLATFILE: "profile.flatfile"
    :vartype PROFILEFLATFILE: str
    :cvar PROFILEXML: "profile.xml"
    :vartype PROFILEXML: str
    :cvar PROFILEJSON: "profile.json"
    :vartype PROFILEJSON: str
    :cvar QUEUE: "queue"
    :vartype QUEUE: str
    :cvar TRADINGPARTNER: "tradingpartner"
    :vartype TRADINGPARTNER: str
    :cvar TPGROUP: "tpgroup"
    :vartype TPGROUP: str
    :cvar TPORGANIZATION: "tporganization"
    :vartype TPORGANIZATION: str
    :cvar TPCOMMOPTIONS: "tpcommoptions"
    :vartype TPCOMMOPTIONS: str
    :cvar WEBSERVICE: "webservice"
    :vartype WEBSERVICE: str
    :cvar WEBSERVICEEXTERNAL: "webservice.external"
    :vartype WEBSERVICEEXTERNAL: str
    :cvar PROCESSROUTE: "processroute"
    :vartype PROCESSROUTE: str
    :cvar CUSTOMLIBRARY: "customlibrary"
    :vartype CUSTOMLIBRARY: str
    :cvar EDISTANDARD: "edistandard"
    :vartype EDISTANDARD: str
    :cvar FLOWSERVICE: "flowservice"
    :vartype FLOWSERVICE: str
    :cvar SCRIPTPROCESSING: "script.processing"
    :vartype SCRIPTPROCESSING: str
    :cvar SCRIPTMAPPING: "script.mapping"
    :vartype SCRIPTMAPPING: str
    :cvar XSLT: "xslt"
    :vartype XSLT: str
    """

    CERTIFICATE = "certificate"
    CONNECTORACTION = "connector-action"
    CONNECTORSETTINGS = "connector-settings"
    CROSSREF = "crossref"
    DOCUMENTCACHE = "documentcache"
    TRANSFORMMAP = "transform.map"
    TRANSFORMFUNCTION = "transform.function"
    CERTIFICATEPGP = "certificate.pgp"
    PROCESS = "process"
    PROCESSPROPERTY = "processproperty"
    PROFILEDB = "profile.db"
    PROFILEEDI = "profile.edi"
    PROFILEFLATFILE = "profile.flatfile"
    PROFILEXML = "profile.xml"
    PROFILEJSON = "profile.json"
    QUEUE = "queue"
    TRADINGPARTNER = "tradingpartner"
    TPGROUP = "tpgroup"
    TPORGANIZATION = "tporganization"
    TPCOMMOPTIONS = "tpcommoptions"
    WEBSERVICE = "webservice"
    WEBSERVICEEXTERNAL = "webservice.external"
    PROCESSROUTE = "processroute"
    CUSTOMLIBRARY = "customlibrary"
    EDISTANDARD = "edistandard"
    FLOWSERVICE = "flowservice"
    SCRIPTPROCESSING = "script.processing"
    SCRIPTMAPPING = "script.mapping"
    XSLT = "xslt"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ComponentType._member_map_.values()))


@JsonMap(
    {
        "branch_id": "branchId",
        "branch_name": "branchName",
        "component_id": "componentId",
        "copied_from_component_id": "copiedFromComponentId",
        "copied_from_component_version": "copiedFromComponentVersion",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "current_version": "currentVersion",
        "folder_id": "folderId",
        "folder_name": "folderName",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
        "sub_type": "subType",
        "type_": "type",
        "encrypted_values": "encryptedValues",
        "process_overrides": "processOverrides",
        "folder_full_path": "folderFullPath",
    }
)
class Component(BaseModel):
    """Component

    :param branch_id: If specified, the branch on which you want to manage the component., defaults to None
    :type branch_id: str, optional
    :param branch_name: branch_name, defaults to None
    :type branch_name: str, optional
    :param component_id: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service., defaults to None
    :type component_id: str, optional
    :param copied_from_component_id: Read only. If you copied the component, this field is the ID of the original component from where you copied the component., defaults to None
    :type copied_from_component_id: str, optional
    :param copied_from_component_version: Read only. If you copied the component, this field is the revision number of the original component you copied., defaults to None
    :type copied_from_component_version: int, optional
    :param created_by: Read only. User name of the user who created the component., defaults to None
    :type created_by: str, optional
    :param created_date: Read only. Date and time., defaults to None
    :type created_date: str, optional
    :param current_version: Read only. Indicates if the value specified in the version field is the most current and latest revision number created for the component on the **Build** tab. A value of True indicates that the revision number is the most current revision number on the **Build** tab, whereas False indicates that the version field value is not the most current revision number., defaults to None
    :type current_version: bool, optional
    :param deleted: Read only. Indicates if the component is deleted. A value of True indicates a deleted status, whereas False indicates an active status., defaults to None
    :type deleted: bool, optional
    :param folder_id: The ID of the folder where the component currently resides., defaults to None
    :type folder_id: str, optional
    :param folder_name: Read only. The folder location of the component within Component Explorer., defaults to None
    :type folder_name: str, optional
    :param modified_by: Read only. User name of the user who last modified the component., defaults to None
    :type modified_by: str, optional
    :param modified_date: Read only. Date and time., defaults to None
    :type modified_date: str, optional
    :param name: Read only., defaults to None
    :type name: str, optional
    :param sub_type: Read only. Used by connector-related components \(connections and operations\) to identify the connector type. Subtype values are the internal connector ID, not the user-facing name.See [Connector object](/api/platformapi#tag/Connector)., defaults to None
    :type sub_type: str, optional
    :param type_: Read only. The type of component. See the section **Component Types** later in this topic for a complete list of component type values, defaults to None
    :type type_: ComponentType, optional
    :param version: Read only., defaults to None
    :type version: int, optional
    :param encrypted_values: encrypted_values, defaults to None
    :type encrypted_values: EncryptedValues, optional
    :param description: If specified, the text description that appears at the top of an opened component.Optional for CREATE and UPDATE., defaults to None
    :type description: str, optional
    :param object: The XML structure of the component object. The structure of the object contents vary by component type. Required. Object name for specific component type. Determines the type of component to create and update. Recommend exporting existing components to determine values.   \>**Note:** These values are slightly different from Component/@type values (reference the [Component Metadata object](/api/platformapi#tag/ComponentMetadata) topic for more information)., defaults to None
    :type object: dict, optional
    :param process_overrides: For process type components, specifies overridden values (for example, variables overridden by environment extensions)., defaults to None
    :type process_overrides: dict, optional
    :param folder_full_path: \<br/\>version \<br/\>type \<br/\>createdDate \<br/\>createdBy \<br/\>modifiedDate \<br/\>modifiedBy \<br/\>Deleted \<br/\>currentVersion \<br/\>folderName \<br/\>folderFullPath.   Read-only system-generated values returned in the response. If included in the response, values for these fields are ignored., defaults to None
    :type folder_full_path: str, optional
    """

    def __init__(
        self,
        branch_id: str = SENTINEL,
        branch_name: str = SENTINEL,
        component_id: str = SENTINEL,
        copied_from_component_id: str = SENTINEL,
        copied_from_component_version: int = SENTINEL,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        current_version: bool = SENTINEL,
        deleted: bool = SENTINEL,
        folder_id: str = SENTINEL,
        folder_name: str = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        name: str = SENTINEL,
        sub_type: str = SENTINEL,
        type_: ComponentType = SENTINEL,
        version: int = SENTINEL,
        encrypted_values: EncryptedValues = SENTINEL,
        description: str = SENTINEL,
        object: dict = SENTINEL,
        process_overrides: dict = SENTINEL,
        folder_full_path: str = SENTINEL,
        # Phase 2: XML preservation fields
        object_xml: str = SENTINEL,
        _object_element: ET.Element = SENTINEL,
        _original_xml: str = SENTINEL,
        **kwargs,
    ):
        """Component

        :param branch_id: If specified, the branch on which you want to manage the component., defaults to None
        :type branch_id: str, optional
        :param branch_name: branch_name, defaults to None
        :type branch_name: str, optional
        :param component_id: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service., defaults to None
        :type component_id: str, optional
        :param copied_from_component_id: Read only. If you copied the component, this field is the ID of the original component from where you copied the component., defaults to None
        :type copied_from_component_id: str, optional
        :param copied_from_component_version: Read only. If you copied the component, this field is the revision number of the original component you copied., defaults to None
        :type copied_from_component_version: int, optional
        :param created_by: Read only. User name of the user who created the component., defaults to None
        :type created_by: str, optional
        :param created_date: Read only. Date and time., defaults to None
        :type created_date: str, optional
        :param current_version: Read only. Indicates if the value specified in the version field is the most current and latest revision number created for the component on the **Build** tab. A value of True indicates that the revision number is the most current revision number on the **Build** tab, whereas False indicates that the version field value is not the most current revision number., defaults to None
        :type current_version: bool, optional
        :param deleted: Read only. Indicates if the component is deleted. A value of True indicates a deleted status, whereas False indicates an active status., defaults to None
        :type deleted: bool, optional
        :param folder_id: The ID of the folder where the component currently resides., defaults to None
        :type folder_id: str, optional
        :param folder_name: Read only. The folder location of the component within Component Explorer., defaults to None
        :type folder_name: str, optional
        :param modified_by: Read only. User name of the user who last modified the component., defaults to None
        :type modified_by: str, optional
        :param modified_date: Read only. Date and time., defaults to None
        :type modified_date: str, optional
        :param name: Read only., defaults to None
        :type name: str, optional
        :param sub_type: Read only. Used by connector-related components \(connections and operations\) to identify the connector type. Subtype values are the internal connector ID, not the user-facing name.See [Connector object](/api/platformapi#tag/Connector)., defaults to None
        :type sub_type: str, optional
        :param type_: Read only. The type of component. See the section **Component Types** later in this topic for a complete list of component type values, defaults to None
        :type type_: ComponentType, optional
        :param version: Read only., defaults to None
        :type version: int, optional
        :param encrypted_values: encrypted_values, defaults to None
        :type encrypted_values: EncryptedValues, optional
        :param description: If specified, the text description that appears at the top of an opened component.Optional for CREATE and UPDATE., defaults to None
        :type description: str, optional
        :param object: The XML structure of the component object. The structure of the object contents vary by component type. Required. Object name for specific component type. Determines the type of component to create and update. Recommend exporting existing components to determine values.   \>**Note:** These values are slightly different from Component/@type values (reference the [Component Metadata object](/api/platformapi#tag/ComponentMetadata) topic for more information)., defaults to None
        :type object: dict, optional
        :param process_overrides: For process type components, specifies overridden values (for example, variables overridden by environment extensions)., defaults to None
        :type process_overrides: dict, optional
        :param folder_full_path: \<br/\>version \<br/\>type \<br/\>createdDate \<br/\>createdBy \<br/\>modifiedDate \<br/\>modifiedBy \<br/\>Deleted \<br/\>currentVersion \<br/\>folderName \<br/\>folderFullPath.   Read-only system-generated values returned in the response. If included in the response, values for these fields are ignored., defaults to None
        :type folder_full_path: str, optional
        """
        if branch_id is not SENTINEL:
            self.branch_id = branch_id
        if branch_name is not SENTINEL:
            self.branch_name = branch_name
        if component_id is not SENTINEL:
            self.component_id = component_id
        if copied_from_component_id is not SENTINEL:
            self.copied_from_component_id = copied_from_component_id
        if copied_from_component_version is not SENTINEL:
            self.copied_from_component_version = copied_from_component_version
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if current_version is not SENTINEL:
            self.current_version = self._define_bool("current_version", current_version, nullable=True)
        if deleted is not SENTINEL:
            self.deleted = self._define_bool("deleted", deleted, nullable=True)
        if folder_id is not SENTINEL:
            self.folder_id = folder_id
        if folder_name is not SENTINEL:
            self.folder_name = folder_name
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if name is not SENTINEL:
            self.name = name
        if sub_type is not SENTINEL:
            self.sub_type = sub_type
        if type_ is not SENTINEL:
            self.type_ = self._enum_matching(type_, ComponentType.list(), "type_")
        if version is not SENTINEL:
            self.version = version
        if encrypted_values is not SENTINEL:
            self.encrypted_values = self._define_object(
                encrypted_values, EncryptedValues
            )
        if description is not SENTINEL:
            self.description = description
        if object is not SENTINEL:
            self.object = object
        if process_overrides is not SENTINEL:
            self.process_overrides = process_overrides
        if folder_full_path is not SENTINEL:
            self.folder_full_path = folder_full_path
        
        # Phase 2: XML preservation fields
        if object_xml is not SENTINEL:
            self.object_xml = object_xml
        if _object_element is not SENTINEL:
            self._object_element = _object_element
        if _original_xml is not SENTINEL:
            self._original_xml = _original_xml
        
        self._kwargs = kwargs

    # ========== Phase 2: XML Preservation Methods ==========
    
    def to_xml(self) -> str:
        """Generate XML from Component model with preserved structure.
        
        This method creates a valid XML representation of the component by:
        1. Starting with the original XML structure (if available)
        2. Updating only the changed fields (name, description, etc.)
        3. Preserving the <object> structure exactly
        
        :return: Valid XML ready to send to the Boomi API
        :rtype: str
        """
        # If we have the original XML, use it as the base
        if hasattr(self, '_original_xml') and self._original_xml:
            root = ET.fromstring(self._original_xml)
        elif hasattr(self, '_object_element') and self._object_element:
            # Create from stored element (this is less common)
            root = self._object_element
        else:
            # Fallback: create minimal XML structure
            ns = "http://api.platform.boomi.com/"
            root = ET.Element(f"{{{ns}}}Component")
            root.set("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
            root.set("xmlns:bns", ns)
        
        # Update component metadata fields
        if hasattr(self, 'component_id') and self.component_id:
            root.set('componentId', self.component_id)
        if hasattr(self, 'name') and self.name:
            root.set('name', self.name)
        if hasattr(self, 'type_') and self.type_:
            type_value = self.type_.value if hasattr(self.type_, 'value') else str(self.type_)
            root.set('type', type_value)
        if hasattr(self, 'folder_id') and self.folder_id:
            root.set('folderId', self.folder_id)
        if hasattr(self, 'folder_name') and self.folder_name:
            root.set('folderName', self.folder_name)
        if hasattr(self, 'folder_full_path') and self.folder_full_path:
            root.set('folderFullPath', self.folder_full_path)
        if hasattr(self, 'version') and self.version:
            root.set('version', str(self.version))
        if hasattr(self, 'created_date') and self.created_date:
            root.set('createdDate', self.created_date)
        if hasattr(self, 'created_by') and self.created_by:
            root.set('createdBy', self.created_by)
        if hasattr(self, 'modified_date') and self.modified_date:
            root.set('modifiedDate', self.modified_date)
        if hasattr(self, 'modified_by') and self.modified_by:
            root.set('modifiedBy', self.modified_by)
        if hasattr(self, 'deleted') and self.deleted is not None:
            root.set('deleted', str(self.deleted).lower())
        if hasattr(self, 'current_version') and self.current_version is not None:
            root.set('currentVersion', str(self.current_version).lower())
        if hasattr(self, 'branch_name') and self.branch_name:
            root.set('branchName', self.branch_name)
        if hasattr(self, 'branch_id') and self.branch_id:
            root.set('branchId', self.branch_id)
        
        # Handle description element
        if hasattr(self, 'description') and self.description:
            ns = "http://api.platform.boomi.com/"
            desc_elem = root.find(f"{{{ns}}}description")
            if desc_elem is None:
                # Create description element after encryptedValues
                encrypted_elem = root.find(f"{{{ns}}}encryptedValues")
                if encrypted_elem is not None:
                    idx = list(root).index(encrypted_elem) + 1
                    desc_elem = ET.Element(f"{{{ns}}}description")
                    root.insert(idx, desc_elem)
                else:
                    desc_elem = ET.SubElement(root, f"{{{ns}}}description")
            desc_elem.text = self.description
        
        # Preserve object XML if available
        if hasattr(self, 'object_xml') and self.object_xml:
            ns = "http://api.platform.boomi.com/"
            # Remove existing object element if present
            obj_elem = root.find(f"{{{ns}}}object")
            if obj_elem is not None:
                root.remove(obj_elem)
            
            # Parse and insert the stored object XML
            try:
                obj_root = ET.fromstring(f'<bns:object xmlns:bns="{ns}">{self.object_xml}</bns:object>')
                # Find the right position (after description, before processOverrides)
                process_overrides = root.find(f"{{{ns}}}processOverrides")
                if process_overrides is not None:
                    idx = list(root).index(process_overrides)
                    root.insert(idx, obj_root)
                else:
                    root.append(obj_root)
            except ET.ParseError:
                # If parsing fails, skip object XML
                pass
        
        # Ensure processOverrides element exists (even if empty)
        ns = "http://api.platform.boomi.com/"
        if root.find(f"{{{ns}}}processOverrides") is None:
            ET.SubElement(root, f"{{{ns}}}processOverrides")
        
        # Register namespaces for proper serialization
        ET.register_namespace("", "http://api.platform.boomi.com/")
        ET.register_namespace("bns", "http://api.platform.boomi.com/")
        
        return ET.tostring(root, encoding='unicode', xml_declaration=True)
    
    @property
    def object_deprecated(self) -> dict:
        """Access to object dict (deprecated).
        
        This property provides backward compatibility but warns users
        that dict-based object access is deprecated in favor of XML preservation.
        
        :return: Object dict for backward compatibility
        :rtype: dict
        """
        if hasattr(self, 'object') and self.object:
            warnings.warn(
                "Component.object as dict is deprecated. "
                "Use object_xml or raw XML methods for reliable updates.",
                DeprecationWarning,
                stacklevel=2
            )
            return self.object
        return {}
    
    def set_object_xml(self, xml: str):
        """Set the object XML content directly.
        
        This method allows you to set the object XML content while
        maintaining the XML structure exactly as provided.
        
        :param xml: XML string for the object content
        :type xml: str
        """
        self.object_xml = xml
        # Also try to parse as ElementTree for potential DOM operations
        try:
            if xml:
                # Store the inner content (without the wrapper)
                if xml.startswith('<') and '>' in xml:
                    self._object_element = ET.fromstring(xml)
        except ET.ParseError:
            # If parsing fails, just store the string
            self._object_element = None




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class InvalidDocumentRouting(Enum):
    """An enumeration representing different categories.

    :cvar DOCUMENTSPATH: "documentsPath"
    :vartype DOCUMENTSPATH: str
    :cvar ERRORSPATH: "errorsPath"
    :vartype ERRORSPATH: str
    """

    DOCUMENTSPATH = "documentsPath"
    ERRORSPATH = "errorsPath"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, InvalidDocumentRouting._member_map_.values())
        )


@JsonMap(
    {
        "expect_ack_for_outbound": "expectAckForOutbound",
        "invalid_document_routing": "invalidDocumentRouting",
        "profile_id": "profileId",
        "qualifier_validation": "qualifierValidation",
        "type_id": "typeId",
        "use999_ack": "use999Ack",
        "use_ta1_ack": "useTA1Ack",
        "validate_outbound_transaction_sets": "validateOutboundTransactionSets",
    }
)
class PartnerDocumentType(BaseModel):
    """PartnerDocumentType

    :param expect_ack_for_outbound: expect_ack_for_outbound, defaults to None
    :type expect_ack_for_outbound: bool, optional
    :param invalid_document_routing: invalid_document_routing, defaults to None
    :type invalid_document_routing: InvalidDocumentRouting, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param profile_id: profile_id, defaults to None
    :type profile_id: str, optional
    :param qualifier_validation: qualifier_validation, defaults to None
    :type qualifier_validation: bool, optional
    :param type_id: type_id, defaults to None
    :type type_id: str, optional
    :param use999_ack: use999_ack, defaults to None
    :type use999_ack: bool, optional
    :param use_ta1_ack: use_ta1_ack, defaults to None
    :type use_ta1_ack: bool, optional
    :param validate_outbound_transaction_sets: validate_outbound_transaction_sets, defaults to None
    :type validate_outbound_transaction_sets: bool, optional
    """

    def __init__(
        self,
        expect_ack_for_outbound: bool = SENTINEL,
        invalid_document_routing: InvalidDocumentRouting = SENTINEL,
        name: str = SENTINEL,
        profile_id: str = SENTINEL,
        qualifier_validation: bool = SENTINEL,
        type_id: str = SENTINEL,
        use999_ack: bool = SENTINEL,
        use_ta1_ack: bool = SENTINEL,
        validate_outbound_transaction_sets: bool = SENTINEL,
        **kwargs
    ):
        """PartnerDocumentType

        :param expect_ack_for_outbound: expect_ack_for_outbound, defaults to None
        :type expect_ack_for_outbound: bool, optional
        :param invalid_document_routing: invalid_document_routing, defaults to None
        :type invalid_document_routing: InvalidDocumentRouting, optional
        :param name: name, defaults to None
        :type name: str, optional
        :param profile_id: profile_id, defaults to None
        :type profile_id: str, optional
        :param qualifier_validation: qualifier_validation, defaults to None
        :type qualifier_validation: bool, optional
        :param type_id: type_id, defaults to None
        :type type_id: str, optional
        :param use999_ack: use999_ack, defaults to None
        :type use999_ack: bool, optional
        :param use_ta1_ack: use_ta1_ack, defaults to None
        :type use_ta1_ack: bool, optional
        :param validate_outbound_transaction_sets: validate_outbound_transaction_sets, defaults to None
        :type validate_outbound_transaction_sets: bool, optional
        """
        if expect_ack_for_outbound is not SENTINEL:
            self.expect_ack_for_outbound = expect_ack_for_outbound
        if invalid_document_routing is not SENTINEL:
            self.invalid_document_routing = self._enum_matching(
                invalid_document_routing,
                InvalidDocumentRouting.list(),
                "invalid_document_routing",
            )
        if name is not SENTINEL:
            self.name = name
        if profile_id is not SENTINEL:
            self.profile_id = profile_id
        if qualifier_validation is not SENTINEL:
            self.qualifier_validation = qualifier_validation
        if type_id is not SENTINEL:
            self.type_id = type_id
        if use999_ack is not SENTINEL:
            self.use999_ack = use999_ack
        if use_ta1_ack is not SENTINEL:
            self.use_ta1_ack = use_ta1_ack
        if validate_outbound_transaction_sets is not SENTINEL:
            self.validate_outbound_transaction_sets = validate_outbound_transaction_sets
        self._kwargs = kwargs




from typing import Any, List
from urllib.parse import quote

from .request import Request
from .utils import extract_original_data
from ...models.utils.sentinel import was_value_set
from ...net.headers.base_header import BaseHeader
from ...net.transport.api_error import ApiError


class Serializer:
    """
    A class for handling serialization of URL components such as headers, cookies, path parameters, and query parameters.

    :ivar str url: The base URL to be serialized.
    :ivar dict[str, str] headers: A dictionary containing headers for the request.
    :ivar list[str] cookies: A list containing cookie strings for the request.
    :ivar dict[str, str] path: A dictionary containing path parameters for the request.
    :ivar list[str] query: A list containing query parameters for the request.
    :ivar dict[int, ApiError] errors: A dictionary of HTTP status codes to error models.
    """

    def __init__(self, url: str, default_headers: List[BaseHeader] = []):
        """
        Initializes a Serializer instance with the base URL.

        :param str url: The base URL to be serialized.
        :param list[BaseHeader] default_headers: A list of default headers to be added to the request. Defaults to an empty list.
        """
        self.url: str = url
        self.headers: dict[str, str] = {}
        self.cookies: list[str] = []
        self.path: dict[str, str] = {}
        self.query: list[str] = []
        self.errors: dict[int, ApiError] = {}

        for header in default_headers:
            for key, value in header.get_headers().items():
                self.add_header(key, value)

    def add_header(
        self, key: str, data: Any, explode: bool = False, nullable: bool = False
    ) -> "Serializer":
        """
        Adds a header to the request.

        :param str key: The header key.
        :param Any data: The data to be serialized as the header value.
        :param bool explode: Flag indicating whether to explode the data.
        :return: The Serializer instance for method chaining.
        :rtype: Serializer
        """
        if not nullable and data is None:
            return self

        if not was_value_set(data):
            return self

        data = extract_original_data(data)

        self.headers[key] = self._serialize_value(
            data, explode=explode, quote_str_values=False
        )
        return self

    def add_cookie(
        self, key: str, data: Any, explode: bool = False, nullable: bool = False
    ) -> "Serializer":
        """
        Adds a cookie to the request.

        :param str key: The cookie key.
        :param Any data: The data to be serialized as the cookie value.
        :param bool explode: Flag indicating whether to explode the data.
        :return: The Serializer instance for method chaining.
        :rtype: Serializer
        """
        if not nullable and data is None:
            return self

        if not was_value_set(data):
            return self

        data = extract_original_data(data)

        self.cookies.append(
            f"{key}={self._serialize_value(data, explode=explode, quote_str_values=False)}"
        )
        return self

    def add_path(
        self,
        key: str,
        data: Any,
        explode: bool = False,
        style: str = "simple",
        nullable: bool = False,
    ) -> "Serializer":
        """
        Adds a path parameter to the request.

        :param str key: The path parameter key.
        :param Any data: The data to be serialized as the path parameter value.
        :param bool explode: Flag indicating whether to explode the data.
        :param str style: The style of serialization for the path parameter.
        :return: The Serializer instance for method chaining.
        :rtype: Serializer
        """
        if not nullable and data is None:
            return self

        if not was_value_set(data):
            return self

        data = extract_original_data(data)

        if style == "simple":
            self.path[key] = self._serialize_value(data=data, explode=explode)
        elif style == "label":
            separator = "." if explode else ","
            self.path[key] = "." + self._serialize_value(
                data=data, explode=explode, separator=separator
            )
        elif style == "matrix":
            separator = ","

            if isinstance(data, list) and explode:
                separator = f";{key}="
            elif explode:
                separator = ";"

            prefix = ";" if isinstance(data, dict) and explode else f";{key}="
            self.path[key] = prefix + self._serialize_value(
                data, explode=explode, separator=separator
            )
        else:
            raise ValueError(f"Unsupported path style: {style}")

        return self

    def add_query(
        self,
        key: str,
        data: Any,
        explode: bool = True,
        style: str = "form",
        nullable: bool = False,
    ) -> "Serializer":
        """
        Adds a query parameter to the request.

        :param str key: The query parameter key.
        :param Any data: The data to be serialized as the query parameter value.
        :param bool explode: Flag indicating whether to explode the data.
        :param str style: The style of serialization for the query parameter.
        :return: The Serializer instance for method chaining.
        :rtype: Serializer
        """
        if not nullable and data is None:
            return self

        if not was_value_set(data):
            return self

        data = extract_original_data(data)

        if style == "form":
            separator = (
                f"&{key}="
                if explode and isinstance(data, list)
                else ("&" if explode else ",")
            )
            prefix = "" if (explode and isinstance(data, dict)) else f"{key}="
            query_param = f"{prefix}{self._serialize_value(data=data, explode=explode, separator=separator)}"
        elif style == "spaceDelimited":
            separator = f"&{key}=" if explode else f"%20"
            query_param = f"{key}={self._serialize_value(data=data, explode=explode, separator=separator)}"
        elif style == "pipeDelimited":
            separator = f"&{key}=" if explode else "|"
            query_param = f"{key}={self._serialize_value(data=data, explode=explode, separator=separator)}"
        elif style == "deepObject":
            query_param = "".join(
                f"{key}[{k}]={quote(self._serialize_value(v))}&"
                for k, v in data.items()
            ).rstrip("&")

        self.query.append(query_param)
        return self

    def add_error(self, status: int, error: ApiError) -> "Serializer":
        """
        Adds an error to the request.

        :param int status: The HTTP status code associated with the error.
        :param ApiError error: The ApiError class representing the error.
        :return: The Serializer instance for method chaining.
        :rtype: Serializer
        """
        if not was_value_set(error):
            return self

        self.errors[status] = error
        return self

    def serialize(self) -> Request:
        """
        Serializes the components and returns a Request object.

        :return: The Request object containing the serialized components.
        :rtype: Request
        """
        final_url = self._define_url()

        if len(self.cookies) > 0:
            self.headers["Cookie"] = ";".join(self.cookies)

        return (
            Request()
            .set_url(final_url)
            .set_headers(self.headers)
            .set_errors(self.errors)
        )

    def _define_url(self) -> str:
        """
        Constructs the final URL by replacing path parameters and appending query parameters.

        :return: The final URL.
        :rtype: str
        """
        final_url = self.url

        for key, value in self.path.items():
            final_url = final_url.replace(f"{{{key}}}", value)

        if len(self.query) > 0:
            final_url += "?" + "&".join(self.query)

        return final_url

    def _serialize_value(
        self,
        data: Any,
        separator: str = ",",
        explode: bool = False,
        quote_str_values: bool = True,
    ) -> str:
        """
        Serializes a value based on the specified separator and explode flag.

        :param Any data: The data to be serialized.
        :param str separator: The separator used for serialization.
        :param bool explode: Flag indicating whether to explode the data.
        :return: The serialized value.
        :rtype: str
        """
        if data is None:
            return "null"

        if isinstance(data, list):
            return separator.join(
                self._serialize_value(item, separator, explode) for item in data
            )

        if isinstance(data, dict):
            if explode:
                return separator.join(
                    [
                        f"{k}={self._serialize_value(v, separator, explode)}"
                        for k, v in data.items()
                    ]
                )
            else:
                return separator.join(
                    [
                        self._serialize_value(item, separator, explode)
                        for sublist in data.items()
                        for item in sublist
                    ]
                )

        if isinstance(data, str) and quote_str_values:
            return quote(data)
        if isinstance(data, bool):
            return str(data).lower()
        if isinstance(data, (int, float)):
            return str(data)

        return data




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_expression import EnvironmentExpression, EnvironmentExpressionGuard
from .environment_simple_expression import EnvironmentSimpleExpression
from .environment_grouping_expression import EnvironmentGroupingExpression


@JsonMap({})
class EnvironmentQueryConfigQueryFilter(BaseModel):
    """EnvironmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentExpression
    """

    def __init__(self, expression: EnvironmentExpression, **kwargs):
        """EnvironmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentExpression
        """
        self.expression = EnvironmentExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentQueryConfig(BaseModel):
    """EnvironmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentQueryConfigQueryFilter
    """

    def __init__(self, query_filter: EnvironmentQueryConfigQueryFilter, **kwargs):
        """EnvironmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .execution_count_account_simple_expression import (
    ExecutionCountAccountSimpleExpression,
)
from .execution_count_account_grouping_expression import (
    ExecutionCountAccountGroupingExpression,
)


class ExecutionCountAccountExpressionGuard(OneOfBaseModel):
    class_list = {
        "ExecutionCountAccountSimpleExpression": ExecutionCountAccountSimpleExpression,
        "ExecutionCountAccountGroupingExpression": ExecutionCountAccountGroupingExpression,
    }


ExecutionCountAccountExpression = Union[
    ExecutionCountAccountSimpleExpression, ExecutionCountAccountGroupingExpression
]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..edi_custom_connector_record import EdiCustomConnectorRecordService
from ...models import (
    EdiCustomConnectorRecordQueryResponse,
    EdiCustomConnectorRecordQueryConfig,
)


class EdiCustomConnectorRecordServiceAsync(EdiCustomConnectorRecordService):
    """
    Async Wrapper for EdiCustomConnectorRecordServiceAsync
    """

    def query_edi_custom_connector_record(
        self, request_body: EdiCustomConnectorRecordQueryConfig = None
    ) -> Awaitable[Union[EdiCustomConnectorRecordQueryResponse, str]]:
        return to_async(super().query_edi_custom_connector_record)(request_body)

    def query_more_edi_custom_connector_record(
        self, request_body: str
    ) -> Awaitable[Union[EdiCustomConnectorRecordQueryResponse, str]]:
        return to_async(super().query_more_edi_custom_connector_record)(request_body)
